import javax.swing.*;
import java.awt.*;
import java.awt.image.*;

/**
 * Die Klasse <CODE>BlockVektor</CODE> stellt einen zeilenweise oder durch
 * Zickzack Scan ausgelesenen <CODE>Block</CODE> dar. Ein <CODE>BlockVektor<CODE>
 * Objekt kann sich in eine RLE kodierte <CODE>RLESymbolfolge</CODE> verwandeln,
 * und als Balkendiagramm darstellen.
 * @author Hari
 * @version 1
 * @see Block
 * @see RLESymbolfolge
 */
public class BlockVektor {
    /**
     * Array mit den 64 "linearisierten" Werten eines 8x8-Blocks
     */
    public int[] werte;

    /**
     * Konstruktor, erzeugt uninitialisiertes Objekt
     * @author Hari
     * @version 1
     */
    public BlockVektor() {
        werte = new int[64];
    }

    /**
     * Erzeugt einen <CODE>BlockVektor</CODE> aus einem <CODE>int</CODE> array
     * @author Hari
     * @version 1
     * @param int[64]
     */
    public BlockVektor(int[] vektor) {
        werte = vektor;
    }

    /**
     * Liefert einen <CODE>SymbolBlock</CODE>, der RLE codiert ist.
     * @author Hari
     * @version 1
     * @param
     * @returns SymbolBlock
     */
    public SymbolBlock getRLECompressed() {
        SymbolBlock symbBlock;
        RLESymbolfolge folge;
        RLESymbol symb;
        int numZeros;
        int numSymbol1, numSymbol2;  // brauchen wir fr berechnung des compressionRatio
        int i;

        // ac
        folge = new RLESymbolfolge();
        numSymbol1 = numSymbol2 = 0;
        i = 1;           // beginnt bei 1, wir wollen ja nur die AC Komponenten
        while (i < 64) {
        	// nullen zhlen, dabei index weiterbewegen
        	numZeros = 0;
        	while (i<64 && werte[i]==0) {
        		i++;
        		numZeros++;
        	}
        	
        	// EndOfBlock?
        	if (i == 64) {
        		symb = new RLESymbol1();    // EOB = symbol1(0,0) senden
        		((RLESymbol1)symb).zeroCount = 0;
        		((RLESymbol1)symb).sizeNonZero = 0;
        		folge.addElement(symb);
        		numSymbol1++;
        	}
        	else {    // kein EOB, also normale sequenz senden        	      	
        	    while (numZeros >= 16) {   // ber 15 nullen am Stck!
        		    numZeros -= 16;
        		    symb = new RLESymbol1();   // wir senden ein symbol1(15,0)
        		    ((RLESymbol1)symb).zeroCount = 15;
        		    ((RLESymbol1)symb).sizeNonZero = 0;
        		    folge.addElement(symb);
        		    numSymbol1++;
        	    }
        	
                 // numZeros ist jetzt >=0 und <=15, wir senden das symbol1.
                symb = new RLESymbol1();
                ((RLESymbol1)symb).zeroCount = numZeros;
                ((RLESymbol1)symb).sizeNonZero = 1;   // auf irgendeinen wert <> 0 setzen (0,0=EOB)
                folge.addElement(symb);
                numSymbol1++;
                // und das symbol2
                symb = new RLESymbol2();
                ((RLESymbol2)symb).amplitude = werte[i];
                folge.addElement(symb);
                numSymbol2++;
                i++;
        	}
        }
        // dc Wert und ac Folge zu einem SymbolBlock zusammenbauen und retournieren
        return new SymbolBlock(werte[0], folge);
    }

    /**
     *
     * @author  Raimund Specht
     * @version 1-13.12.1999
     * @param   width
     * @param   height
     * @param   c
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage paintBar(int width, int height, Color c) {
        final int balkenBreiteRelativ = 1;  // Verhltnis Balkenbreite zu Zwischenraumbreite
        int i, maxValue;
        float faktor, balkenBreite, zwischenraumBreite;
        BufferedImage image;
        Graphics2D g2D;
        
        image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        g2D = image.createGraphics();
        g2D.setColor(GUIParameter.INACTIVE_BACKGROUND);
        g2D.fillRect(0, 0, width, height);
        
        maxValue=0;  // betragsmig grten Wert bestimmen
        for (i=0; i<64; i++)
            if (Math.abs(werte[i]) > maxValue) maxValue=Math.abs(werte[i]);
        if (maxValue > 0) {
            faktor = (float)(height-10) / (float)maxValue;
        } else {
            faktor = 0;
        }
        zwischenraumBreite = (float)width / (64*balkenBreiteRelativ + 63);
        balkenBreite = zwischenraumBreite * balkenBreiteRelativ;
        
        for (i=0; i<64; i++) {
            float xPos;
            Polygon poly;
            
            poly=new Polygon();
            xPos = i*(balkenBreite+zwischenraumBreite);
            poly.addPoint((int)xPos, height-5);
            poly.addPoint((int)(xPos + balkenBreite), height-5);
            poly.addPoint((int)(xPos + balkenBreite), (int)(height-5-Math.abs(werte[i])*faktor));
            poly.addPoint((int)xPos, (int)(height-5-Math.abs(werte[i])*faktor));
            g2D.setColor(c);
            g2D.fillPolygon(poly);
            g2D.setColor(Color.black);
            g2D.drawPolygon(poly);
        }
        
        return image;
    }
}