import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.image.*;

/**
 * Die Klasse <CODE>FloatBlock</CODE> kapselt einen 8x8-Block eines Bildes mit
 * <CODE>float</CODE> Elementen, also nach der DCT. <CODE>FloatBlock</CODE> kann sich
 * in einen <CODE>Block</CODE>quantisieren und in einer <CODE>JTable</CODE> darstellen.
 * Auerdem verfgt <CODE>FloatBlock</CODE> noch ber die Fhigkeit, sich als 3D-Balkendiagramm darzustellen.
 * @author  Raimund Specht/Hari
 * @see     Block
 */
public class FloatBlock extends Matrix {
	public float matrix[][];
	
	/**
	 * Konstruktor fr die <CODE>FloatBlock</CODE> Klasse
     * Erzeugt uninitialisiertes Objekt
	 * @author Hari
	 * @version 1 - 4.1.00
	 * @returns FloatBlock
	 */
	public FloatBlock() {
		matrix = new float[8][8];
	}
	
    /**
     * Diese Methode fhrt die Quantisierung des Blocks durch und liefert das Ergebnis als neues <CODE>Block</CODE>-Objekt
     * zurck. Das Originalobjekt bleibt unverndert.
     * @author  Raimund Specht, Hari
     * @version 2 - 4.1.00
     * @param   qMatrix Die zu verwendende Quantisierungsmatrix
     * @returns Block
     */
    public Block getQuantisiert(Matrix qMatrix) {
        int i, j;
        Block blk = new Block();

        for (i=0; i<8; i++)
            for (j=0; j<8; j++)
                blk.matrix[i][j] = Math.round(matrix[i][j] / qMatrix.matrix[i][j]);
        return blk;
    }
    
    /**
     * Konvertiert den <CODE>FloatBlock</CODE> zu einem <CODE>Block</CODE>,
     * rundet also die Matrizeneintrge.
     * @author Hari
     * @version 1 - 4.1.00
     * @returns Block
     */
    public Block toIntBlock() {
        Block block = new Block();
        for (int x=0; x<8; x++)
        	for (int y=0; y<8; y++)
        		block.matrix[x][y] = Math.round(matrix[x][y]);
        return block;
    }
    
    /**
     * Zeichnet die Werte des Blocks als 3D-Balkendiagramm. Erzeugt einen
     * <CODE>Block</CODE> und ruft die zugehrige Methode auf.
     * @author  Hari
     * @version 1 - 4.1.00
     * @param   width Gewnschte Breite des erzeugten 3D-Balkendiagramms
     * @param   height Gewnschte Hhe des erzeugten 3D-Balkendiagramms
     * @param   Color Gewnschte Farbe der Balken
     * @returns BufferedImage
     */
    public BufferedImage paintBar3D(int width, int height, Color c) {
    	return toIntBlock().paintBar3D(width, height, c);
    }

    /**
     * Implementiert das von <CODE>Matrix</CODE> geerbte Interface <CODE>TableModel</CODE>.
     * @author  Raimund Specht, Christoph Ballhause
     * @version 2-24.01.2000
     * @param   rowIndex Zeile der angesprochenen Zelle
     * @param   columnIndex Spalte der angesprochenen Zelle
     * @returns Object
     * @see     Matrix
     * @see     javax.swing.table.TableModel
     */
    public Object getValueAt(int rowIndex, int columnIndex) { 
        return new Integer(Math.round(matrix[columnIndex][rowIndex]));
    }

    /**
     * Mit diese Methode kann fr eine beliebige Zelle berprft werden, ob diese Zelle vom Benutzer
     * verndert werden kann, wenn der Block in einer <CODE>JTable</CODE> dargestellt wird. Die Klasse
     * <CODE>Block</CODE> liefert hier immer <CODE><B>false</B></CODE> zurck, d.h. es kann keine Zelle
     * bearbeitet werden.
     * @author  Raimund Specht
     * @version 1-03.12.1999
     * @param   rowIndex Zeile der angesprochenen Zelle
     * @param   columnIndex Spalte der angesprochenen Zelle
     * @returns boolean
     * @see     javax.swing.table.TableModel
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return false;
    }

    /**
     *
     * @author  Raimund Specht
     * @version 1-02.12.1999
     * @param   color Die gewnschte Hintergrundfarbe der <CODE>JTable</CODE>
     * @returns java.swing.JTable
     */
    public JTable paintMatrix(Color farbe) {
        JTable table;
        
        table = new JTable(this);
        table.setBackground(farbe);
        table.setFont(GUIParameter.SMALL_FONT);
        table.setPreferredSize(new Dimension(192, table.getPreferredSize().height));
        table.setMaximumSize(new Dimension(192, table.getPreferredSize().height));
        table.setMinimumSize(new Dimension(192, table.getPreferredSize().height));
        return table;
    }
}