/**
 * Die <CODE>Histogramm</CODE> Klasse speichert, naja, wir httens fast
 * geahnt, ein Histogramm. Histogramme werden einerseits dazu benutzt,
 * optimierte Huffman Tabellen herzustellen (mit <CODE>buildHuffmanTable</CODE>)
 * und andererseiz um Bitlngenverteilungen grafisch darzustellen (ber
 * <CODE>bitlaengenHistogramm</CODE>).
 * Mann kan Histogramme mit <CODE>addHisto</CODE> zusammenfgen.
 * @author  Hari
 * @version 1 - 4.2.00
 */
import java.awt.*;
import java.awt.image.*;

/**
 *
 * @author Hari
 * @version 1
 */
public class Histogramm {
    public int werte[];
    
    /**
     * Legt ein neues <CODE>Histogramm</CODE>-Objekt
     * der Gre <CODE>groesse</CODE>an und initialisiert es mit Nullen
     * @author  Hari
     * @version 1
     * @param   int
     */
    public Histogramm(int groesse) {
    	int i;
    	
    	werte = new int[groesse];
    	for (i=0; i<groesse; i++)
    		werte[i] = 0;
    }
    
    /**
     * Fgt ein anderes Histogramm zu diesem hinzu (Werte addieren sich).
     * Die Lngen werden als gleich angenommen.
     * @author  Hari
     * @version 1
     * @param   Histogramm
     */
    public void addHisto(Histogramm histo) {
    	for (int i=0; i<werte.length; i++)
    		werte[i] += histo.werte[i];
    }
    
    /**
     * Stellt die JPEG konforme (auf 16 bit Symbollnge beschrnkte) Huffman
     * Tabelle fr das Histogramm auf
     * @author Hari
     * @version 2 - 3.1.00
     * @returns OurHuffmanTable
     */
    public OurHuffmanTable buildHuffmanTable() {
        OurHuffmanTable huffTable;
        int i;
        int freq[];
        int freq_backup[];
        int v1, v2, min;
        int others[];
        int codesize[];  // Lnge des Huffman Codes fr ein Symbol
        boolean done;
        
        // codesize[] ausrechnen
        others = new int[257];
        codesize = new int[257];
        for (i=0; i<257; i++) {
            others[i] = -1;
            codesize[i] = 0;
        }
        freq = werte;
        freq_backup = (int[])freq.clone();
        
        do {
            min = 100000;
            v1 = 0;
            for (i=0; i<257; i++)
                if (freq[i]>0 && freq[i]<=min) {
                    min = freq[i];
                    v1 = i;
                }
            v2 = -1;
            min = 100000;
            for (i=0; i<257; i++)
                if (freq[i]>0 && i!=v1 && freq[i]<min) {
                    min = freq[i];
                    v2 = i;
                }
            if (v2==-1)
                break;

            freq[v1] += freq[v2];
            freq[v2] = 0;
            
            done = false;
            do {
                codesize[v1]++;
                if (others[v1]==-1)
                    done = true;
                else
                    v1 = others[v1];
            } while (!done);
            others[v1] = v2;
            
            done = false;
            do {
                codesize[v2]++;
                if (others[v2]==-1)
                    done = true;
                else
                    v2 = others[v2];
            } while (!done);
        } while (true);
        
        // aus codesize[] nun bits[] ausrechnen, darin steht dann die
        // Anzahl Codewrter fr eine Codewortlnge
        int bits[];
        
        bits = new int[33];
        for (i=0; i<32; i++)
            bits[i] = 0;
        for (i=0; i<257; i++)
            if (codesize[i] != 0)
                bits[codesize[i]]++;
        
        // bits[] auf 16 bit beschrnken
        int j;
        
        i = 32;
        while (true) {
            if (bits[i]>0) {
                j = i-1;
                do
                    j--;
                while (bits[j]==0);
                bits[i] -= 2;
                bits[i-1]++;
                bits[j+1] += 2;
                bits[j]--;
            }
            else {
                i--;
                if (i==16)
                    break;
            }
        }
        while (bits[i]==0)
            i--;
        bits[i]--;
        
        // Hufigkeitstabelle freq wieder herstellen
        freq = (int[])freq_backup.clone();
        
        // Nach Hufigkeit aufsteigende Reihenfolge von freq in freqOrder sortieren
        int freqOrder[];
        freqOrder = new int[256];
        for (i=0; i<256; i++)
            freqOrder[i] = i;
        do {
            done = true;
            for (i=0; i<256-1; i++)
                if (freq[freqOrder[i]]<freq[freqOrder[i+1]]) {
                    int temp = freqOrder[i];
                    freqOrder[i] = freqOrder[i+1];
                    freqOrder[i+1] = temp;
                    done = false;
                }
        } while (!done);

        // aus bits[] und freqOrder[] die Lngentabelle herstellen
        int symbIndex;
        
        huffTable = new OurHuffmanTable();
        symbIndex = 0;
        for (i=0; i<=16; i++)
            for (j=0; j<bits[i]; j++) {
                huffTable.lengths[freqOrder[symbIndex]] = i;
                symbIndex++;
            }
        
        // Huffman Symbole erzeugen
        int code = 0;
        symbIndex = 0;
        for (i=0; i<=16; i++) {
            for (j=0; j<bits[i]; j++) {
                huffTable.symbols[freqOrder[symbIndex]] = code;
                code++;
                symbIndex++;
            }
            code <<= 1;
        }
        
        // JPEG konforme Tabellen (fr Ausgabe) noch herstellen
        huffTable.codedLengths = new short[17];
        for (i=0; i<17; i++)
            huffTable.codedLengths[i] = (short)bits[i];
        huffTable.codedSymbols = new short[symbIndex];  // in symbIndex steht noch die Anzahl Symbole von ein paar Zeilen vorher!
        symbIndex = 0;
        for (i=0; i<=16; i++)
            for (j=0; j<bits[i]; j++) {
                huffTable.codedSymbols[symbIndex] = (short)freqOrder[symbIndex];
                symbIndex++;
            }
        return huffTable;
    }
    
    /**
     * Diese Methode malt das Histogramm als Balkendiagramm und gibt ein
     * <CODE>BufferedImage</CODE> zurck.
     * Die Durchschnittslnge der Werte wird rechts oben draufgeschrieben.
     * @author Hari
     * @version 3 - 16.1.00
     * @param int width
     * @param int height
     * @param Color
     * @returns BufferedImage
     */
    public BufferedImage bitlaengenHistogramm(int width, int height, Color c) {
        BufferedImage image;
        Graphics2D g2D;
        FontMetrics fm;
        int i, x, balkenBreite, luecke;
        int max;
        float faktor;
        int randUnten = 20;
        int randOben = 5;
        int randLinx;
        int randRechz;
        long summeLaengen, summeAnzahl;
        int schriftPos;
        int freieBalken;
        int stringBreite, schriftHoehe;
        int breiteInBalken;
        int durchschnLaenge;
        String schnittText;
        boolean gefunden;
        
        image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        g2D = image.createGraphics();
        g2D.setColor(GUIParameter.INACTIVE_BACKGROUND);
        g2D.fillRect(0, 0, width, height);
        fm = g2D.getFontMetrics();
        schriftHoehe = fm.getHeight();
        
        max = 0;
        summeLaengen = 0;
        summeAnzahl = 0;
        for (i=0; i<werte.length; i++) {
        	if (werte[i] > max)
        		max = werte[i];
            summeLaengen += werte[i]*i;
            summeAnzahl += werte[i];
        }
        durchschnLaenge = (int)(10*summeLaengen/summeAnzahl);
            
        // beschriftung linx
        String maxProzentStr = Integer.toString((int)(100*max/summeAnzahl));
        maxProzentStr = maxProzentStr.concat("%");
        randLinx = 5+2+5+fm.stringWidth(maxProzentStr);
        g2D.setColor(Color.black);
        g2D.drawLine(randLinx-1, randOben, randLinx-1, height-randUnten);
        g2D.drawLine(randLinx-5, height-randUnten, randLinx-1, height-randUnten);
        g2D.drawLine(randLinx-5, randOben, randLinx-1, randOben);
        g2D.drawString("0%", randLinx-fm.stringWidth("0%")-2-5, height-randUnten+fm.getHeight()/2);
        g2D.drawString(maxProzentStr, 5, randOben+fm.getHeight()/2);
        
        // rand ausrechnen, pfeil & beschriftung rechz untn (ende der x axe)
        String laengeStr = new String("Bitlnge");
        randRechz = fm.stringWidth(laengeStr) + 5+5+5;  // +5 fr den Pfeil, 5 frn rand, 5 fr zwischen Pfeil&Schrift
        Polygon pfeil = new Polygon();
        pfeil.addPoint(width-randRechz, height-randUnten-3);
        pfeil.addPoint(width-randRechz, height-randUnten+3);
        pfeil.addPoint(width-randRechz+5, height-randUnten);
        g2D.fillPolygon(pfeil);
        g2D.drawString(laengeStr, width-randRechz-1+10, height-randUnten+schriftHoehe/2);
        
        faktor = (float)(height-randOben-randUnten) / max;
        balkenBreite = (width-randLinx-randRechz) / (2*werte.length);
        luecke = (width-randLinx-randRechz) / (2*werte.length);
        
        // beschriftung untn
        x = randLinx;
        for (i=0; i<werte.length; i++) {
            String s = Integer.toString(i);
            g2D.drawString(s, x+(balkenBreite-fm.stringWidth(s))/2, height-5);
            x += balkenBreite+luecke;
        }
        g2D.drawLine(randLinx, height-randUnten, width-randRechz-1, height-randUnten);
        randUnten++;  // damit die balken eins ber der Linie beginnen

        // breite des durchschnitts textes in balken ausrechnen
        schnittText = new String("Durchschn. Lnge: ");
        schnittText = schnittText.concat(Integer.toString(durchschnLaenge/10));
        schnittText = schnittText.concat(".");
        schnittText = schnittText.concat(Integer.toString(durchschnLaenge%10));
        stringBreite = fm.stringWidth(schnittText);
        breiteInBalken = 0;
        while (breiteInBalken * (balkenBreite+luecke) < stringBreite)
            breiteInBalken++;
        
        // balken zeichnen
        x = randLinx;
        freieBalken = 0;
        gefunden = false;
        schriftPos = width-5-stringBreite;
        for (i=0; i<werte.length; i++) {
            int yOben;
            if (werte[i] > 0) {
                Polygon poly = new Polygon();
                
                yOben = height-randUnten-(int)(werte[i]*faktor);
                poly.addPoint(x, height-randUnten);
                poly.addPoint(x+balkenBreite, height-randUnten);
                poly.addPoint(x+balkenBreite, yOben);
                poly.addPoint(x, yOben);
                g2D.setColor(c);
                g2D.fillPolygon(poly);
                g2D.setColor(Color.black);
                g2D.drawPolygon(poly);
        	}
            else
                yOben = height-randUnten;  // oder so
            if (yOben > randOben+schriftHoehe) {
                freieBalken++;
                if (freieBalken==breiteInBalken) {
                    schriftPos = x-(breiteInBalken-1)*(balkenBreite+luecke);
                    gefunden = true;
                    freieBalken--;
                }
            }
            else
                freieBalken = 0;
	        x += balkenBreite+luecke;
        }
        
        g2D.drawString(schnittText, schriftPos, schriftHoehe);
        
        return image;
    }
}