import java.net.*;
import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.lang.*;
import java.util.*;
import com.sun.image.codec.jpeg.*;

/**
 * Die Klasse <CODE>JPEGImage</CODE> reprsentiert ein Bild und stellt Methoden zur Verfgung,
 * ein Bild aus einer Datei bzw. ber das Internet zu laden,
 * das Bild in die RGB-Farbkanle zu zerlegen, in den YIQ-Farbraum zu konvertieren, einzelne
 * 8x8-Blcke auszulesen sowie das komplette Bild JPEG-komprimiert darzustellen.
 * @author  Raimund Specht, Hari
 */
public class JPEGImage implements ImageObserver {
    /**
     * Speichert das Bild (direkt nach dem Laden noch in Originalgre)
     */
    private BufferedImage image;
	/**
	 * Speichert das jungfruliche Bild, wie es nach dem Laden aussieht
	 */
    private BufferedImage rohImage;
    /**
     * Zwischenspeicher (cache) fr das rote Teilbild (sobald berechnet, sonst <CODE><B>null</B></CODE>)
     */
    private BufferedImage rImage;
    /**
     * Zwischenspeicher (cache) fr das grne Teilbild (sobald berechnet, sonst <CODE><B>null</B></CODE>)
     */
    private BufferedImage gImage;
    /**
     * Zwischenspeicher (cache) fr das blaue Teilbild (sobald berechnet, sonst <CODE><B>null</B></CODE>)
     */
    private BufferedImage bImage;
    /**
     * Zwischenspeicher (cache) fr das Y-Teilbild (sobald berechnet, sonst <CODE><B>null</B></CODE>)
     */
    private BufferedImage yImage;
    /**
     * Zwischenspeicher (cache) fr das I-Teilbild (sobald berechnet, sonst <CODE><B>null</B></CODE>)
     */
    private BufferedImage iImage;
    /**
     * Zwischenspeicher (cache) fr das Q-Teilbild (sobald berechnet, sonst <CODE><B>null</B></CODE>)
     */
    private BufferedImage qImage;
    /**
     * Zwischenspeicher (cache) fr das gesubsampelte I-Teilbild
     */
    private BufferedImage subsampledIImage;
    /**
     * Zwischenspeicher (cache) fr das gesubsampelte Q-Teilbild
     */
    private BufferedImage subsampledQImage;
    
    /**
     * 2 Merker fr Parameter beim letzten Aufruf von getIImage/getQImage.
     * Es werden nur die Felder <CODE>hSubsample</CODE> bzw. <CODE>vSubsample</CODE>,
     * sowie <CODE>filename</CODE> und <CODE>skalierungsQualitaet</CODE> benutzt.
     */
    private JPEGParameter iImageLastParams = new JPEGParameter();
    private JPEGParameter qImageLastParams = new JPEGParameter();
    
    /**
     * Die Abmessungen des Bildes ohne den schwarzen Balken, der
     * durch die Skalierung des geladenen Bildes entsteht
     */
    private int firstActLine, lastActLine, firstActCol, lastActCol;
    	
    /**
     * Zwischenspeicher (cache) fr das mit einer Markierung des aktuellen 8x8-Blocks 
     * versehene Y-Teilbild (sobald berechnet, sonst <CODE><B>null</B></CODE>)
     */
    private BufferedImage markeredImage;
    
    /**
     * Je ein Cache fr das dekomprimierte JPEG Bild und das Differenzbild
     */
    private BufferedImage decomprRGBImage;
    private BufferedImage diffImage;
    
    /**
     * Merker fr Parameter beim letzten Aufruf von getDifferenceImage/getDecompressedJPEGImage/
     * quantBloeckeErzeugen/symboleErzeugen/symboleCodieren
     */
    private JPEGParameter differenceImageParameter, jpegImageParameter;
    private JPEGParameter quantBloeckeErzeugenParams, symboleErzeugenParams, symboleCodierenParams;
    
    /**
     * Position des markierten Blockes in Pixels, nicht in Blcken!
     */
    private int markerX, markerY;
    
    /**
     * Gre des fertig codierten JPEG Bildes
     */
    private int codedSize;
    
    /**
     * Lnge des JPEG Datenstromes (also codedSize minus Lnge der Headers)
     */
    private int lengthHuffmanCoded;

    /**
     * Der <CODE>HuffmanWriter</CODE>, der das kodierte JPG Bild aufnimmt
     * und spter auf Pladde schreibt, wenn es des Benutzers Wunsch ist
     */
    private HuffmanWriter hWriter;
    
    /*
     * Die 8x8 Blcke des Bildes nach der Quantisierung
     */
    private Vector quantYBloecke, quantIBloecke, quantQBloecke;

    /**
     * Zum Umkehren der Block-Umordnung beim Erzeugen der MCUs
     * Wird fr die Darstellung des aktuellen Blockes aufm Entropie Panel
     * gebraucht (da sind die Blcke schon umgeordnet, nach Huffman)
     */
    private int deMCUOrder[];
    
    /**
     * Die DC Luminanzwerte des Bildes, wie dcLumSymbole, nur als int Array
     */
    private int dcLumWerte[];
    /**
     * Die DC Chrominanzwerte des Bildes, wie dcChromSymbole, nur als int Array
     */
    private int dcChromWerte[];

    private OurHuffmanTable acLumHuffmanTable, acChromHuffmanTable, dcLumHuffmanTable, dcChromHuffmanTable;
    
    private SymbBlockFolge yBlockFolge, crBlockFolge, cbBlockFolge;
    private int indexAktuellerBlock;
    
    /**
     * Erzeugt <CODE>image</CODE> neu, mit der angegebenen Verschiebung.
     * Wird also nur im Scrollmodus verwendet.
     * Setzt die Parameter <CODE>xShift</CODE> und <CODE>yShift</CODE> im
     * <CODE>JPEGParameter</CODE> Objekt
     * @author Hari
     * @version 1 - 10.1.2000
     * @param int xShift
     * @param int yShift
     */
	public void setImageShift(int xShift, int yShift) {
		int newXShift, newYShift;
		int rohWidth, rohHeight;
		
		rohWidth = rohImage.getWidth();
		rohHeight = rohImage.getHeight();
		newXShift = FrameVisJPEG.vis.parameter.xShift - xShift;
		newYShift = FrameVisJPEG.vis.parameter.yShift - yShift;
 		if (newXShift > rohWidth-192) newXShift = rohWidth-192;
		if (newYShift > rohHeight-144) newYShift = rohHeight-144;
		if (newXShift < 0) newXShift = 0;
		if (newYShift < 0) newYShift = 0;
		
		FrameVisJPEG.vis.parameter.xShift = newXShift;
		FrameVisJPEG.vis.parameter.yShift = newYShift;
        rebuildFromRaw(true, newXShift, newYShift);
	}
	
    /**
     * Liefert die Breite des Rohbildes (des geladenen Bildes)
     * @author Hari
     * @returns int
     * @version 1 - 10.1.2000
     */
	public int getRohWidth() {
		return rohImage.getWidth();
	}
	
    /**
     * Liefert die Hhe des Rohbildes (des geladenen Bildes)
     * @author Hari
     * @returns int
     * @version 1 - 10.1.2000
     */
	public int getRohHeight() {
		return rohImage.getHeight();
	}
	
	/**
	 * Macht die Caches fr die Teilbilder ungltig
     * Wird eigentlich nur von auerhalb der Klasse aufgerufen
	 * @author Hari
	 * @version 1 - 10.1.2000
	 */
	public void invalidate() {
        rImage = null;
        gImage = null;
        bImage = null;
        yImage = null;
        iImage = null;
        qImage = null;
        subsampledIImage = null;
        subsampledQImage = null;
        markeredImage = null;
        differenceImageParameter = null;
        jpegImageParameter = null;
        quantBloeckeErzeugenParams = null;
        symboleErzeugenParams = null;
        symboleCodierenParams = null;
	}
	
    /**
     * Der Konstruktor ldt das Bild, das durch die URL identifiziert wird.
     * Das Bild wird automatisch auf eine passende Gre skaliert. Sollte
     * beim Laden ein Fehler auftreten, wird stattdessen ein schwarzes Bild
     * mit einer Fehlermeldung erzeugt.
     * ber <CODE>scrollingEnabled</CODE> wird angegeben, ob der Scroll- oder
     * Skalierungsmodus gelten soll.
     * @author  Raimund Specht
     * @version 2-06.12.1999
     * @param   url Die URL, von der das Bild geladen werden soll
     */
    public JPEGImage(URL url, boolean scrollingEnabled) {
    	Image img;
    	Graphics2D g2D;
    	
        img = JPEG.applet.getImage(url);
        loadImage(img, scrollingEnabled);
    }

    /**
     * Dieser Konstruktor ldt ein Bild aus einer Datei auf der Festplatte.
     * Das Bild wird automatisch auf eine passende Gre skaliert. Sollte
     * beim Laden ein Fehler auftreten, wird stattdessen ein schwarzes Bild
     * mit einer Fehlermeldung erzeugt.
     * ber <CODE>scrollingEnabled</CODE> wird angegeben, ob der Scroll- oder
     * Skalierungsmodus gelten soll.
     * @author  Raimund Specht
     * @version 2-06.12.1999
     * @param   file Datei, aus der das Bild geladen werden soll
     */
    public JPEGImage(String file, boolean scrollingEnabled) {
    	Image img = null;
    	Graphics2D g2D;
    	
        if (JPEG.isApplication) {
            img = Toolkit.getDefaultToolkit().createImage(file);
        } else {
            try {
                img = JPEG.applet.getImage(new URL(JPEG.appletCodeBase, file));
            } catch (MalformedURLException ex) { }
        }
        loadImage(img, scrollingEnabled);
    }
    
    /**
     * Gibt Anzahl der Blcke in einem Y/I/Q Teilbild in horizontaler
     * Richtung zurck.
     * Der Parameter <CODE>resamplingRatio</CODE> sagt, welches
     * Farb Subsampling vorliegt.
     * @author  Hari
     * @version 3 - 8.1.00
     * @param int
     */
    private int xBlockx(int resamplingRatio) {
    	return (image.getWidth()-1) / (8*resamplingRatio) + 1;
    }
    
    /**
     * Gibt Anzahl der Blcke in einem Y/I/Q Teilbild in vertikaler
     * Richtung zurck.
     * Der Parameter <CODE>resamplingRatio</CODE> sagt, welches
     * Farb Subsampling vorliegt.
     * @author  Hari
     * @version 3 - 8.1.00
     * @param int
     */
    private int yBlockx(int resamplingRatio) {
    	return (image.getHeight()-1) / (8*resamplingRatio) + 1;
    }
    
    /**
     * Private Hilfsmethode, die ein Bild komplett ldt und bei einem Fehler
     * stattdessen ein "Fehlermeldungsbild" erzeugt.
     * ber <CODE>scrollingEnabled</CODE> wird angegeben, ob der Scroll- oder
     * Skalierungsmodus gelten soll.
     * @author  Raimund Specht, Hari
     * @version 2 - 10.1.2000
     * @param   newImage Das <CODE>Image</CODE>-Objekt, das geladen werden soll
     */
    private void loadImage(Image newImage, boolean scrollingEnabled) {
        Graphics2D g2D;
        MediaTracker mTracker;
        Canvas c = new Canvas();

        /* mit einem MediaTracker auf das Ende des Ladevorgangs warten */
        mTracker = new MediaTracker(c);
        mTracker.addImage(newImage, 0);
        try {
            mTracker.waitForAll(); 
        } catch(InterruptedException e) {}
        
/**/    if (newImage.getWidth(this) > 0 && newImage.getHeight(this) > 0) {
            this.rohImage = new BufferedImage(newImage.getWidth(this), newImage.getHeight(this), BufferedImage.TYPE_INT_RGB);
            g2D = this.rohImage.createGraphics();
            g2D.drawImage(newImage, 0, 0, this);
/**/    }

/**/    if (mTracker.isErrorID(0) || newImage.getWidth(this) <= 0 || newImage.getHeight(this) <= 0) {
            /* Fehler: "Fehlermeldungsbild" erzeugen */
            image = new BufferedImage(192, 144, BufferedImage.TYPE_INT_RGB);
        
            g2D = image.createGraphics();
            g2D.setColor(Color.white);
            g2D.drawString("Bild kann nicht geladen werden !", 5, 48);
            g2D.drawString("Image cannot be loaded !", 5, 96);
        } else
            // alles ok: image aus rohImage herstellen
			rebuildFromRaw(scrollingEnabled, (new JPEGParameter()).xShift, (new JPEGParameter()).yShift);
    }
	
	/**
	 * Erzeugt das Originalbild (<CODE>image</CODE>) aus dem geladenen
	 * Bild (<CODE>rohImage</CODE>).
	 * Der Parameter <CODE>scrollingEnabled</CODE> sagt, ob der
	 * Scroll- oder der Skalierungsmodus verwendet werden soll.
     * ber die Parameter <CODE>xShift</CODE> und <CODE>yShift</CODE>
     * wird angegeben, um wieviel das Bild verschoben gezeichnet
     * werden soll.
	 * Die vier Variablen, die die Position des schwarzen Balkens
	 * beschreiben, werden je nachdem direkt oder ber den Aufruf
	 * von <CODE>scale</CODE> gesetzt.
	 * @author Hari
	 * @version 1 - 10.1.2000
	 * @param boolean scrollingEnabled
	 */
	public void rebuildFromRaw(boolean scrollingEnabled, int xShift, int yShift) {
	    Graphics2D g2D;
        int xDrawPos, yDrawPos;
        int drawWidth, drawHeight;
        
		if (scrollingEnabled) {
            if (rohImage.getWidth()<192 || rohImage.getHeight()<144) {
                // Jetzt mssen wir eine sackbescheuerte Fallunterscheidung machen,
                // weil das Bild ja zu breit und gleichzeitig zu niedrig, usw. sein kann.
                // Wenixtens funzt es!!!!
                if (rohImage.getWidth()>192) {
                    firstActCol = 0;
                    lastActCol = 191;
                    xDrawPos = 0;
                    drawWidth = 192;
                }
                else {
                    firstActCol = (192-rohImage.getWidth())/2;
                    lastActCol = firstActCol + rohImage.getWidth() - 1;
                    xDrawPos = firstActCol;
                    drawWidth = rohImage.getWidth()-1;
                }
                if (rohImage.getHeight()>144) {
                    firstActLine = 0;
                    lastActLine = 143;
                    yDrawPos = 0;
                    drawHeight = 144;
                }
                else {
                    firstActLine = (144-rohImage.getHeight())/2;
                    lastActLine = firstActLine + rohImage.getHeight() - 1;
                    yDrawPos = firstActLine;
                    drawHeight = rohImage.getHeight()-1;
                }
                image = new BufferedImage(192, 144, BufferedImage.TYPE_INT_RGB);
                g2D = image.createGraphics();
                g2D.drawImage(rohImage.getSubimage(xShift, yShift, drawWidth, drawHeight), xDrawPos, yDrawPos, Color.black, this);
			}
            else {
                image = rohImage.getSubimage(xShift, yShift, 192, 144);
                firstActCol = 0;
                firstActLine = 0;
                lastActCol = 191;
                lastActLine = 143;
            }
		}
		else {
            // Die vier intz firstActCol, usw. werden in diesem Fall von scale gesetzt
			image = new BufferedImage(rohImage.getWidth(), rohImage.getHeight(), BufferedImage.TYPE_INT_RGB);
			g2D = image.createGraphics();
			g2D.drawImage(rohImage, 0, 0, this);
			if (rohImage.getWidth()!=192 || rohImage.getHeight()!=144)
				image = scale(192, 144);
			else {
				firstActCol = 0;
				firstActLine = 0;
				lastActCol = 191;
				lastActLine = 143;
			}
		}
	}
	
    /**
     * Die <CODE>sinc</CODE> Funktion
     * @author  Hari
     * @version 1 - 5.1.00
     * @param   double
     * @returns double
     */
	private float sinc(float x) {
		if (x == 0.0)
			return(1);
		else
			return (float)(Math.sin(3.14159*x) / (3.14159*x));
	}
	
    /**
     * Der <CODE>Lanczos3</CODE> Interpolationskern
     * @author  Hari
     * @version 1 - 5.1.00
     * @param   float
     * @returns float
     */
	private float lanczos3(float x) {
		final float oneThird = (float)1 / 3;
		if (x>-3 && x<3)
			return(sinc(x) * sinc(x*oneThird));
		else
			return(0);
	}
	
    /**
     * Skaliert ein Bild auf die angegebene Breite/Hhe. Das
     * Seitenverhltnis des Bildes wird verndert, falls ntig.
     * Als Kernel wird <CODE>Lanczos3</CODE> benutzt.
     * Eine Lookup Tabelle fr den Kernel bringt brigens keinen merklichen
     * Geschwindigkeitsvorteil, ich habs ausprobiert.
     * Ich hab die Methode so gut wie nicht kommentiert, weil eh keiner
     * wissen will, wie das funktioniert. Oder?
     * @author  Hari
     * @version 1 - 5.1.00
     * @param   BufferedImage
     * @param   int destWidth
     * @param   int destHeight
     * @returns java.awt.image.BufferedImage
     */
	private BufferedImage scaleForgetAboutAspectRatio(BufferedImage image, int destWidth, int destHeight)
	{
		BufferedImage destImage;
		final int filterRadius = 3;  // lanczos3(x)=0 for |x|>3
		int filterRadiusHoriz, filterRadiusVert;
		int srcHeight, srcWidth;
		int srcx, srcy;
		int destx, desty;
		int srcyCenter, srcyBottom, srcyTop;
		int srcxCenter, srcxLeft, srcxRight;
		int weightsHoriz[][];
		int weightsVert[];
		int rval, gval, bval;
		int color;
		int weight;
		int sumWeights;
		long weightFactor;
		int tempImg[][][];
		int i, x, y;
		float desty2srcy[];
		float kernelStretchHoriz, kernelStretchVert;
		int srcArray[][][];

		srcHeight = image.getHeight();
		srcWidth = image.getWidth();
		if (destWidth < srcWidth)
			kernelStretchHoriz = (float)destWidth / srcWidth;
		else
			kernelStretchHoriz = 1;
		filterRadiusHoriz = (int)(filterRadius/kernelStretchHoriz+1);
		if (destHeight < srcHeight)
			kernelStretchVert = (float)destHeight / srcHeight;
		else
			kernelStretchVert = 1;
		filterRadiusVert = (int)(filterRadius/kernelStretchVert+1);
		
		srcArray = new int[srcWidth][srcHeight][3];
		for (x=0; x<srcWidth; x++)
			for (y=0; y<srcHeight; y++) {
					color = image.getRGB(x, y);
					srcArray[x][y][0] = (color&0xff0000)>>16;
					srcArray[x][y][1] = (color&0x00ff00)>>8;
					srcArray[x][y][2] = color&0x0000ff;
			}
		
		tempImg = new int[destWidth][srcHeight][3];
		
		destImage = new BufferedImage(destWidth, destHeight, BufferedImage.TYPE_INT_RGB);
		weightsHoriz = new int[srcWidth][destWidth];
		// pre-calc weights
		for (srcx=0; srcx<srcWidth; srcx++)
			for(destx=0; destx<destWidth; destx++)
				weightsHoriz[srcx][destx] = (int)(lanczos3((((float)(srcWidth-1))*destx/(destWidth-1) - srcx)*kernelStretchHoriz)*kernelStretchHoriz*65536);
		
		for (y=0; y<srcHeight; y++) {
			for (destx=0; destx<destWidth; destx++) {
				srcxCenter = (int)((float)(srcWidth-1)*destx/(destWidth-1));
				if (srcxCenter < filterRadiusHoriz)
					srcxLeft = 0;
				else
					srcxLeft = srcxCenter - filterRadiusHoriz;
				if (srcxCenter > srcWidth-filterRadiusHoriz)
					srcxRight = srcWidth;
				else
					srcxRight = srcxCenter + filterRadiusHoriz;
				rval = 0;
				gval = 0;
				bval = 0;
			
				// make sure sum of weights = 1 to avoid distortion near edges
				sumWeights = 0;
				for (srcx=srcxLeft; srcx<srcxRight; srcx++)
					sumWeights += weightsHoriz[srcx][destx];
				weightFactor = ((long)1) << 32;
				weightFactor /= sumWeights;
			
				for (srcx=srcxLeft; srcx<srcxRight; srcx++) {
					long w = (long)weightsHoriz[srcx][destx];
					w *= weightFactor;
					weight = (int)(w/65536);
					rval += srcArray[srcx][y][0] * weight;
					gval += srcArray[srcx][y][1] * weight;
					bval += srcArray[srcx][y][2] * weight;
				}
				if (rval>255<<16) rval = 255<<16;
				if (rval<0      ) rval = 0;
				if (gval>255<<16) gval = 255<<16;
				if (gval<0      ) gval = 0;
				if (bval>255<<16) bval = 255<<16;
				if (bval<0      ) bval = 0;
				tempImg[destx][y][0] = rval >> 16;
				tempImg[destx][y][1] = gval >> 16;
				tempImg[destx][y][2] = bval >> 16;
			}
		}
		
		weightsVert = new int[srcHeight];
		desty2srcy = new float[destHeight];
		// pre-calc desty->srcy conv table
		for (desty=0; desty<destHeight; desty++)
			desty2srcy[desty] = ((float)(srcHeight-1)*desty/(destHeight-1));			
		
		for (desty=0; desty<destHeight; desty++) {
			// pre-calc weights
			for (srcy=0; srcy<srcHeight; srcy++)
				weightsVert[srcy] = (int)(lanczos3((desty2srcy[desty] - srcy)*kernelStretchVert)*kernelStretchVert*65536);
			for (x=0; x<destWidth; x++) {
				rval = 0;
				gval = 0;
				bval = 0;
				srcyCenter = (int)desty2srcy[desty];
				if (srcyCenter < filterRadiusVert)
					srcyBottom = 0;
				else
					srcyBottom = srcyCenter - filterRadiusVert;
				if (srcyCenter > srcHeight-filterRadiusVert)
					srcyTop = srcHeight;
				else
					srcyTop = srcyCenter + filterRadiusVert;
		
				// make sure sum of weights = 1 to avoid distortion near edges
				sumWeights = 0;
				for (srcy=srcyBottom; srcy<srcyTop; srcy++)
					sumWeights += weightsVert[srcy];
				weightFactor = ((long)1) << 32;
				weightFactor /= sumWeights;
		
				for (srcy=srcyBottom; srcy<srcyTop; srcy++) {
					long w = (long)weightsVert[srcy];
					w *= weightFactor;
					weight = (int)(w/65536);
					rval += tempImg[x][srcy][0] * weight;
					gval += tempImg[x][srcy][1] * weight;
					bval += tempImg[x][srcy][2] * weight;
				}
				if (rval>255<<16) rval = 255<<16;
				if (rval<0      ) rval = 0;
				if (gval>255<<16) gval = 255<<16;
				if (gval<0      ) gval = 0;
				if (bval>255<<16) bval = 255<<16;
				if (bval<0      ) bval = 0;
				rval >>= 16;
				gval >>= 16;
				bval >>= 16;
				color = ((int)rval<<16) + ((int)gval<<8) + ((int)bval);
				destImage.setRGB(x, desty, color);
			}
		}
		return destImage;
	}

	// alte Routine, noch mit floats statt ints
/*	private BufferedImage scaleForgetAboutAspectRatio(BufferedImage image, int destWidth, int destHeight)
	{
		BufferedImage destImage;
		final int filterRadius = 3;  // lanczos3(x)=0 for |x|>3
		int filterRadiusHoriz, filterRadiusVert;
		int srcHeight, srcWidth;
		int srcx, srcy;
		int destx, desty;
		int srcyCenter, srcyBottom, srcyTop;
		int srcxCenter, srcxLeft, srcxRight;
		float weightsHoriz[][];
		float weightsVert[];
		float rval, gval, bval;
		int color;
		float weight;
		float sumWeights;
		float tempImg[][][];
		int i, x, y;
		float desty2srcy[];
		float kernelStretchHoriz, kernelStretchVert;
		int srcArray[][][];
	
		srcHeight = image.getHeight();
		srcWidth = image.getWidth();
		if (destWidth < srcWidth)
			kernelStretchHoriz = (float)destWidth / srcWidth;
		else
			kernelStretchHoriz = 1;
		filterRadiusHoriz = (int)(filterRadius/kernelStretchHoriz+1);
		if (destHeight < srcHeight)
			kernelStretchVert = (float)destHeight / srcHeight;
		else
			kernelStretchVert = 1;
		filterRadiusVert = (int)(filterRadius/kernelStretchVert+1);
		
		srcArray = new int[srcWidth][srcHeight][3];
		for (x=0; x<srcWidth; x++)
			for (y=0; y<srcHeight; y++) {
					color = image.getRGB(x, y);
					srcArray[x][y][0] = (color&0xff0000)>>16;
					srcArray[x][y][1] = (color&0x00ff00)>>8;
					srcArray[x][y][2] = color&0x0000ff;
			}
		
		tempImg = new float[destWidth][srcHeight][3];
		
		destImage = new BufferedImage(destWidth, destHeight, BufferedImage.TYPE_INT_RGB);
		weightsHoriz = new float[srcWidth][destWidth];
		// pre-calc weights
		for (srcx=0; srcx<srcWidth; srcx++)
			for(destx=0; destx<destWidth; destx++)
				weightsHoriz[srcx][destx] = lanczos3((((float)(srcWidth-1))*destx/(destWidth-1) - srcx)*kernelStretchHoriz)*kernelStretchHoriz;
		
		for (y=0; y<srcHeight; y++) {
			for (destx=0; destx<destWidth; destx++) {
				srcxCenter = (int)((float)(srcWidth-1)*destx/(destWidth-1));
				if (srcxCenter < filterRadiusHoriz)
					srcxLeft = 0;
				else
					srcxLeft = srcxCenter - filterRadiusHoriz;
				if (srcxCenter > srcWidth-filterRadiusHoriz)
					srcxRight = srcWidth;
				else
					srcxRight = srcxCenter + filterRadiusHoriz;
				rval = 0;
				gval = 0;
				bval = 0;
			
				// make sure sum of weights = 1 to avoid distortion near edges
				sumWeights = 0;
				for (srcx=srcxLeft; srcx<srcxRight; srcx++)
					sumWeights += weightsHoriz[srcx][destx];
				sumWeights = 1 / sumWeights;
			
				for (srcx=srcxLeft; srcx<srcxRight; srcx++) {
					weight = weightsHoriz[srcx][destx] * sumWeights;
					rval += srcArray[srcx][y][0] * weight;
					gval += srcArray[srcx][y][1] * weight;
					bval += srcArray[srcx][y][2] * weight;
				}
				if (rval>255) rval = 255;
				if (rval<0  ) rval = 0;
				if (gval>255) gval = 255;
				if (gval<0  ) gval = 0;
				if (bval>255) bval = 255;
				if (bval<0  ) bval = 0;
				tempImg[destx][y][0] = rval;
				tempImg[destx][y][1] = gval;
				tempImg[destx][y][2] = bval;
			}
		}
		
		weightsVert = new float[srcHeight];
		desty2srcy = new float[destHeight];
		// pre-calc desty->srcy conv table
		for (desty=0; desty<destHeight; desty++)
			desty2srcy[desty] = ((float)(srcHeight-1)*desty/(destHeight-1));			
		
		for (desty=0; desty<destHeight; desty++) {
			// pre-calc weights
			for (srcy=0; srcy<srcHeight; srcy++)
				weightsVert[srcy] = lanczos3((desty2srcy[desty] - srcy)*kernelStretchVert)*kernelStretchVert;
			for (x=0; x<destWidth; x++) {
				rval = 0;
				gval = 0;
				bval = 0;
				srcyCenter = (int)desty2srcy[desty];
				if (srcyCenter < filterRadiusVert)
					srcyBottom = 0;
				else
					srcyBottom = srcyCenter - filterRadiusVert;
				if (srcyCenter > srcHeight-filterRadiusVert)
					srcyTop = srcHeight;
				else
					srcyTop = srcyCenter + filterRadiusVert;
		
				// make sure sum of weights = 1 to avoid distortion near edges
				sumWeights = 0;
				for (srcy=srcyBottom; srcy<srcyTop; srcy++)
					sumWeights += weightsVert[srcy];
				sumWeights = 1 / sumWeights;
		
				for (srcy=srcyBottom; srcy<srcyTop; srcy++) {
					weight = weightsVert[srcy] * sumWeights;
					rval += tempImg[x][srcy][0] * weight;
					gval += tempImg[x][srcy][1] * weight;
					bval += tempImg[x][srcy][2] * weight;
				}
				if (rval>255) rval = 255;
				if (rval<0  ) rval = 0;
				if (gval>255) gval = 255;
				if (gval<0  ) gval = 0;
				if (bval>255) bval = 255;
				if (bval<0  ) bval = 0;
				color = ((int)rval<<16) + ((int)gval<<8) + ((int)bval);
				destImage.setRGB(x, desty, color);
			}
		}
		return destImage;
	}*/
	
    /**
     * Diese Methode skaliert das Bild auf die angegebene Breite/Hhe.
     * Das Seitenverhltnis bleibt gleich; falls ntig, werden schwarze
     * Rnder angefgt. Ruft <CODE>scaleForgetAboutAspectRatio</CODE> auf.
     * Es wird auch <CODE>firstActCol</CODE>, <CODE>firstActLine</CODE>,
     * <CODE>lastActCol</CODE> und <CODE>lastActLine</CODE> gesetzt.
     * @author  Hari
     * @version 1 - 5.1.00
     * @param   int
     * @param   int
     * @returns java.awt.image.BufferedImage
     */
	private BufferedImage scale(int containerWidth, int containerHeight) {
		BufferedImage destImage, containerImage;
		int srcWidth, srcHeight, destWidth, destHeight;
		float aspectRatio;
		Graphics2D g2D;
		
		srcWidth = image.getWidth();
		srcHeight = image.getHeight();
		if (srcWidth==containerWidth && srcHeight==containerHeight) {
			destImage = new BufferedImage(containerWidth, containerHeight, BufferedImage.TYPE_INT_RGB);
			g2D = destImage.createGraphics();
			g2D.drawImage(image, 0, 0, this);
			firstActCol = 0;
			firstActLine = 0;
			lastActCol = srcWidth-1;
			lastActLine = srcHeight-1;
			return destImage;
		}
			
		containerImage = new BufferedImage(containerWidth, containerHeight, BufferedImage.TYPE_INT_RGB);
		g2D = containerImage.createGraphics();
		aspectRatio = (float)containerWidth / containerHeight;
		if (srcWidth > srcHeight*aspectRatio) {  // zu breit
			destWidth = containerWidth;
			destHeight = (int)(containerWidth*srcHeight/srcWidth);
			destImage = scaleForgetAboutAspectRatio(image, destWidth, destHeight);
    	g2D.drawImage(destImage, 0, (containerHeight-destHeight)/2, Color.black, this);
			firstActCol = 0;
			firstActLine = (containerHeight-destHeight)/2;
			lastActCol = 191;
			lastActLine = firstActLine + destHeight - 1;
		}
		else {  // zu hoch
			destWidth = (int)(containerHeight*srcWidth/srcHeight);
			destHeight = containerHeight;
			destImage = scaleForgetAboutAspectRatio(image, destWidth, destHeight);
    	g2D.drawImage(destImage, (containerWidth-destWidth)/2, 0, Color.black, this);
			firstActCol = (containerWidth-destWidth)/2;
			firstActLine = 0;
			lastActCol = firstActCol + destWidth - 1;
			lastActLine = 143;
		}
		return containerImage;
	}

    /**
     * Mit dieser Methode wird das Interface <CODE>ImageObserver</CODE>
     * implementiert. Die Methode liefert so lange <CODE><B>true</B></CODE> zurck, bis alle Bilddaten geladen sind.
     * @author  Raimund Specht
     * @version 1-06.12.1999
     * @returns boolean
     */
    public boolean imageUpdate(Image img, int infoflags, int x, int y,
                               int width, int height) {
        return (infoflags & ALLBITS) != ALLBITS;
    }
                               
    /**
     * Diese Methode liefert das "rohe", "unbehandelte" Bild zurck, wie es aus der Datei bzw.
     * von der URL gelesen wurde (nach der Skalierung).
     * @author  Raimund Specht
     * @version 1-04.12.1999
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getImage() {
        return image;
    }
	
    /**
     * Diese Methode liefert den roten Farbanteil des Bildes zurck. Falls der rote Farbanteil
     * noch nicht berechnet wurde, wird das Bild zuvor in die drei RGB-Farbkanle zerlegt.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getRImage() {
        if (rImage == null) splitToRGB();
        return rImage;
    }

    /**
     * Diese Methode liefert den grnen Farbanteil des Bildes zurck. Falls der grne Farbanteil
     * noch nicht berechnet wurde, wird das Bild zuvor in die drei RGB-Farbkanle zerlegt.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getGImage() {
        if (gImage == null) splitToRGB();
        return gImage;
    }

    /**
     * Diese Methode liefert den blauen Farbanteil des Bildes zurck. Falls der blaue Farbanteil
     * noch nicht berechnet wurde, wird das Bild zuvor in die drei RGB-Farbkanle zerlegt.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getBImage() {
        if (bImage == null) splitToRGB();
        return bImage;
    }

    /**
     * Diese Methode liefert die Y-Farbkomponente des Bildes zurck. Falls die Y-Komponente
     * noch nicht berechnet wurde, wird das Bild zuvor nach YIQ transformiert.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getYImage() {
        if (yImage == null) splitToYIQ();
        return yImage;
    }

    /**
     * Diese Methode liefert die Y-Farbkomponente des Bildes zurck; der aktuell gewhlte
     * 8x8-Block wird darin durch eine Hervorhebung markiert. Falls ntig, werden die bentigten
     * Daten (Y-Teilbild und markiertes Bild) zuvor berechnet.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getMarkedYImage() {
        if (markeredImage == null) {
            if (yImage == null) splitToYIQ();
            markeredImage = markBlock(yImage);
        }
        return markeredImage;
    }

    /**
     * Diese Methode liefert die gesubsamplete I-Farbkomponente des Bildes zurck.
     * @author  Raimund Specht, Hari
     * @version 2 - 8.1.2000
     * @param   subsample Faktor, um den das Bild gesubsamplet (skaliert) werden soll, z.B. 1, 2 oder 4
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getIImage(int hsubsample, int vsubsample) {
        if (iImage==null)
            splitToYIQ();
        if (subsampledIImage==null || iImageLastParams.hSubsample!=hsubsample || iImageLastParams.vSubsample!=vsubsample || FrameVisJPEG.vis.parameter.filename.compareTo(iImageLastParams.filename)!=0 || FrameVisJPEG.vis.parameter.skalierungsQualitaet!=iImageLastParams.skalierungsQualitaet) {
            subsampledIImage = subsample(iImage, hsubsample, vsubsample);
            iImageLastParams = (JPEGParameter)FrameVisJPEG.vis.parameter.clone();
            return subsampledIImage;
        }
        else {
            BufferedImage newImage = new BufferedImage(subsampledIImage.getWidth(), subsampledIImage.getHeight(), BufferedImage.TYPE_BYTE_GRAY);
            Graphics g2D = newImage.createGraphics();
            g2D.drawImage(subsampledIImage, 0, 0, this);
            return newImage;
        }
    }

    /**
     * Diese Methode liefert die gesubsamplete Q-Farbkomponente des Bildes zurck.
     * @author  Raimund Specht, Hari
     * @version 2 - 8.1.2000
     * @param   subsample Faktor, um den das Bild gesubsamplet (skaliert) werden soll, z.B. 1, 2 oder 4
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getQImage(int hsubsample, int vsubsample) {
        if (qImage==null)
            splitToYIQ();
        if (subsampledQImage==null || qImageLastParams.hSubsample!=hsubsample || qImageLastParams.vSubsample!=vsubsample || FrameVisJPEG.vis.parameter.filename.compareTo(qImageLastParams.filename)!=0 || FrameVisJPEG.vis.parameter.skalierungsQualitaet!=qImageLastParams.skalierungsQualitaet) {
            subsampledQImage = subsample(qImage, hsubsample, vsubsample);
            qImageLastParams = (JPEGParameter)FrameVisJPEG.vis.parameter.clone();
            return subsampledQImage;
        }
        else {
            BufferedImage newImage = new BufferedImage(subsampledQImage.getWidth(), subsampledQImage.getHeight(), BufferedImage.TYPE_BYTE_GRAY);
            Graphics g2D = newImage.createGraphics();
            g2D.drawImage(subsampledQImage, 0, 0, this);
            return newImage;
        }
    }
	
    /**
     * Diese Methode setzt den durch den Benutzer ausgewhlten 8x8-Block.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     * @param   x Eine x-Koordinate im zu markierenden Block (Wertebereich: 0 - 192). Die Ausrichtung
     * auf 8x8-Grenzen erfolgt automatisch.
     * @param   y Eine y-Koordinate im zu markierenden Block (Wertebereich: 0 - 144). Die Ausrichtung
     * auf 8x8-Grenzen erfolgt automatisch.
     */
    public void setMarker(int x, int y) {
        markerX = (x/8) * 8;
        markerY = (y/8) * 8;
        markeredImage = null;
    }
	
    /**
     * Diese Methode zerlegt das Bild in die drei Grundfarben Rot, Grn und Blau. Fr jede der drei
     * Grundfarben wird ein <CODE>BufferedImage</CODE> erzeugt.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     */
    private void splitToRGB() {
        int x, y, c;
        
        rImage = new BufferedImage(192, 144, BufferedImage.TYPE_INT_RGB);
        gImage = new BufferedImage(192, 144, BufferedImage.TYPE_INT_RGB);
        bImage = new BufferedImage(192, 144, BufferedImage.TYPE_INT_RGB);
        
        for (x=0; x<192; x++)
            for (y=0; y<144; y++) {
                c = image.getRGB(x, y);
                rImage.setRGB(x, y, c & 0xFF0000);
                gImage.setRGB(x, y, c & 0x00FF00);
                bImage.setRGB(x, y, c & 0x0000FF);
            }
    }

    /**
     * Diese Methode konvertiert das Bild von RGB in den YIQ-Farbraum. Jede der drei entstehenden
     * Farbkomponenten wird in einem eigenen Schwarz-wei-<CODE>BufferedImage</CODE> abgelegt.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     */
    private void splitToYIQ() {
        int x, y, c, red, green, blue, yColor, iColor, qColor;
        
        yImage = new BufferedImage(192, 144, BufferedImage.TYPE_BYTE_GRAY);
        iImage = new BufferedImage(192, 144, BufferedImage.TYPE_BYTE_GRAY);
        qImage = new BufferedImage(192, 144, BufferedImage.TYPE_BYTE_GRAY);
        
        for (x=0; x<192; x++)
            for (y=0; y<144; y++) {
                c = image.getRGB(x, y);
                // R-G-B-Werte ermitteln
                red = (c & 0xFF0000) >> 16;
                green = (c & 0x00FF00) >> 8;
                blue = c & 0x0000FF;    
                // Y-I-Q berechnen und I-Q normieren
                yColor = (int)(0.299*red + 0.587*green + 0.114*blue);  
                iColor = 128 + (int)(-0.1687*red - 0.3313*green + 0.5*blue);  
                qColor = 128 + (int)(0.5*red - 0.4187*green - 0.0813*blue);   
                
                // Bildpunkte in neuen Teilbildern setzen
                yImage.setRGB(x, y, yColor*0x10101);
                iImage.setRGB(x, y, (new Color(iColor, iColor, iColor)).getRGB());
                qImage.setRGB(x, y, (new Color(qColor, qColor, qColor)).getRGB());
            }
    }
	
    /**
     * Diese Methode zeichnet die Markierung fr einen vom Benutzer ausgewhlten Block in
     * ein Bild ein. Dabei bleibt das Originalbild unverndert; es wird ein neues
     * <CODE>BufferedImage</CODE> zurckgeliefert.
     * @author  Raimund Specht
     * @version 2-06.12.1999
     * @param   image Das Bild, in das die Markierung eingetragen werden soll
     * @returns java.awt.image.BufferedImage
     */
    private BufferedImage markBlock(BufferedImage image) {
        BufferedImage newImage;
        Graphics2D g2D;

        // neues Bild erzeugen (RGB)
        newImage = new BufferedImage(192, 144, BufferedImage.TYPE_INT_RGB);
        g2D = newImage.createGraphics();
        // altes Bild in neues Bild kopieren
        g2D.drawImage(image, 0, 0, this);
        // Marker-Rechteck einzeichnen
        g2D.setColor(Color.yellow);
        g2D.drawRect(markerX, markerY, 7, 7);
        
        return newImage;
    }
	
    /**
     * Diese Methode reintegriert drei Y-, I- und Q-Teilbilder zu einem RGB-Bild. Fr das I- und das Q-Teilbild
     * kann noch je ein Subsampling-Faktor angegeben werden.
     * @author  Raimund Specht, Hari
     * @version 1-07.12.1999
     * @param   yImg Das Y-Teilbild.
     * @param   iImg Das I-Teilbild.
     * @param   qImg Das Q-Teilbild.
     * @param   hSubsample Der horizontale Subsamplingfaktor fr das I- und Q-Teilbild (1 = kein Subsampling, 2 = auf halbe Gre usw.)
     * @param   vSubsample Der vertikale Subsamplingfaktor fr das I- und Q-Teilbild (1 = kein Subsampling, 2 = auf halbe Gre usw.)
     * @returns java.awt.image.BufferedImage
     */
    private BufferedImage yiqToRGB(BufferedImage yImg, BufferedImage iImg, BufferedImage qImg, int hSubsample, int vSubsample) {
        BufferedImage combinedImage;
        int x, y, red, green, blue, yColor, iColor, qColor;
        
        combinedImage = new BufferedImage(yImg.getWidth(), yImg.getHeight(), BufferedImage.TYPE_INT_RGB);
        iImg = supersample(iImg);
        qImg = supersample(qImg);

        for (x=0; x<192; x++)
            for (y=0; y<144; y++) {
                yColor = yImg.getRGB(x, y) & 0xFF;
                iColor = (iImg.getRGB(x, y) & 0xFF) - 128;
                qColor = (qImg.getRGB(x, y) & 0xFF) - 128;

                red = (int)(yColor + 1.402*qColor);
                green = (int)(yColor - 0.34414*iColor - 0.71414*qColor);
                blue = (int)(yColor + 1.772*iColor);

                // RGB-Werte mssen zwischen 0 und 255 liegen (Rundungsfehler korrigieren):
                if (red < 0) red = 0;
                if (red > 255) red = 255;
                if (green < 0) green = 0;
                if (green > 255) green = 255;
                if (blue < 0) blue = 0;
                if (blue > 255) blue = 255;
                
                combinedImage.setRGB(x, y, (new Color(red, green, blue)).getRGB());
            }
        
        return combinedImage;
    }
	
    /**
     * Schnellste Variante des Subsamplings, dafr schlechteste Qualitt.
     * Die neue Gre ergibt sich aus der von <CODE>this.image</CODE>.
     * @author  Raimund Specht, Hari
     * @version 1 - 9.1.2000
     * @param   image
     * @returns java.awt.image.BufferedImage
     */
	private BufferedImage subsampleLowQual(BufferedImage image, int hsubsample, int vsubsample) {
        int newWidth, newHeight;
        BufferedImage newImage;
        Graphics2D g2D;
        int x, x2, y, y2, wert;
        
        newWidth = image.getWidth() / hsubsample;
        newHeight = image.getHeight() / vsubsample;
        newImage = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_BYTE_GRAY);
        
        for (x=0; x<newWidth; x++)
            for (y=0; y<newHeight; y++)
                for (x2=x*hsubsample; x2<(x+1)*hsubsample; x2++)
                    for (y2=y*vsubsample; y2<(y+1)*vsubsample; y2++) {
                    	wert = image.getRGB(x2, y2) & 0xFF;
                    	newImage.setRGB(x, y, (wert<<16)+(wert<<8)+wert);
                    }
        
		return newImage;
	}
		
    /**
     * Mittlere Variante des Subsamplings, bezglich Geschwindigkeit und Qualitt.
     * Die neue Gre ergibt sich aus der von <CODE>this.image</CODE>.
     * @author  Raimund Specht, Hari
     * @version 1 - 9.1.2000
     * @param   image
     * @returns java.awt.image.BufferedImage
     */
	private BufferedImage subsampleMediumQual(BufferedImage image, int hsubsample, int vsubsample) {
        int newWidth, newHeight;
        BufferedImage newImage;
        Graphics2D g2D;
        int x, x2, y, y2, summe;
        
        newWidth = image.getWidth() / hsubsample;
        newHeight = image.getHeight() / vsubsample;
        newImage = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_BYTE_GRAY);
        
        for (x=0; x<newWidth; x++)
            for (y=0; y<newHeight; y++) {
                summe = 0;
                for (x2=x*hsubsample; x2<(x+1)*hsubsample; x2++)
                    for (y2=y*vsubsample; y2<(y+1)*vsubsample; y2++)
                        summe += image.getRGB(x2, y2) & 0xFF;
                        
                summe /= hsubsample*vsubsample;
                newImage.setRGB(x, y, (new Color(summe, summe, summe)).getRGB());
            }
        
		return newImage;
	}

    /**
     * Langsameste Variante des Subsamplings, dafr beste Qualitt.
     * Die neue Gre ergibt sich aus der von <CODE>this.image</CODE>.
     * Als Argument kriegt die Methode ein Grauwert Bild bergeben, es wird dann
     * <CODE>scaleForgetAboutAspectRatio</CODE> aufgerufen, das eigentlich
     * auf RGB Bildern (nicht auf Graubildern) arbeitet, aber es klappt trotzdem,
     * und eine Skalierungsroutine extra fr Graubilder wr auch nicht schneller,
     * ich hab das mal ausprobiert.
     * @author  Hari
     * @version 1 - 9.1.2000
     * @param   image
     * @returns java.awt.image.BufferedImage
     */
 	private BufferedImage subsampleHighQual(BufferedImage image, int hsubsample, int vsubsample) {
 		int newWidth, newHeight;
 		BufferedImage newImage;
 		
 		newWidth = image.getWidth() / hsubsample;
 		newHeight = image.getHeight() / vsubsample;
 		return scaleForgetAboutAspectRatio(image, newWidth, newHeight);
    }
    
   /**
     * Diese Methode fhrt das (Farb-)Subsampling durch; dabei wird ein neues <CODE>BufferedImage</CODE> 
     * zurckgeliefert, das alte bleibt unverndert. 
     * Es wird entweder <CODE>subsampleLowQual</CODE>, <CODE>subsampleMediumQual</CODE>
     * oder <CODE>subsampleHighQual</CODE> aufgerufen.
     * @author  Hari
     * @version 3 - 7.1.2000
     * @param   image
     * @param   subsample Subsampling-Verhltnis / -Faktor (1 = kein Subsampling, 2 = auf halbe Gre, usw.)
     * @returns java.awt.image.BufferedImage
     */
    private BufferedImage subsample(BufferedImage image, int hsubsample, int vsubsample) {
        if (hsubsample==1 && vsubsample==1) {
        	int newHeight = image.getHeight();
        	int newWidth = image.getWidth();
        	BufferedImage newImage = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_BYTE_GRAY);
	        Graphics2D g2D = newImage.createGraphics();
	        g2D.drawImage(image, 0, 0, this);
	        return newImage;
        }
        
    	if (FrameVisJPEG.vis.parameter.skalierungsQualitaet == 0)
    		return subsampleLowQual(image, hsubsample, vsubsample);
    	else if (FrameVisJPEG.vis.parameter.skalierungsQualitaet == 1)
    		return subsampleMediumQual(image, hsubsample, vsubsample);
    	else
    		return subsampleHighQual(image, hsubsample, vsubsample);
    }
    
    /**
     * Schnellste Variante des Supersamplings, dafr schlechteste Qualitt.
     * Die neue Gre ergibt sich aus der von <CODE>this.image</CODE>.
     * Der Resampling Faktor muss ganzzahlig sein.
     * @author  Raimund Specht, Hari
     * @version 1 - 9.1.2000
     * @param   image
     * @returns java.awt.image.BufferedImage
     */
	private BufferedImage supersampleLowQual(BufferedImage image) {
        int newWidth, newHeight;
        BufferedImage newImage;
        Graphics2D g2D;
        int x, y, wert;
        int xfaktor, yfaktor;
        
        newWidth = this.image.getWidth();
        newHeight = this.image.getHeight();
        xfaktor = newWidth / image.getWidth();
        yfaktor = newHeight / image.getHeight();
        newImage = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_BYTE_GRAY);
        
        for (x=0; x<newWidth; x++)
            for (y=0; y<newHeight; y++) {
            	wert = image.getRGB(x/xfaktor, y/yfaktor) & 0xFF;
            	newImage.setRGB(x, y, (wert<<16)+(wert<<8)+wert);
            }
        
		return newImage;
	}
		
    /**
     * Mittlere Variante des Subsamplings, bezglich Geschwindigkeit und Qualitt.
     * Die neue Gre ergibt sich aus der von <CODE>this.image</CODE>.
     * Horizontaler und vertikaler Sampling Faktor muss eine Zweierpotenz sein.
     * @author  Hari
     * @version 1 - 9.1.2000
     * @param   image
     * @returns java.awt.image.BufferedImage
     */
    private BufferedImage supersampleMediumQual(BufferedImage image) {
        int newWidth, newHeight;
        BufferedImage newImage;
        Graphics2D g2D;
        int x, y;
        int xfaktor, xfaktor2;
        int yfaktor, yfaktor2;
        int xf2shift, yf2shift;
        int zeilen[][];
        int i, j;
        int p, q;
        int val;
        int sum, incr;
        int newX, newY;
        
        newWidth = this.image.getWidth();
        newHeight = this.image.getHeight();
        xfaktor = newWidth / image.getWidth();
        xfaktor2 = 2*xfaktor;
        yfaktor = newWidth / image.getHeight();
        yfaktor2 = 2*yfaktor;
        zeilen = new int[xfaktor2][2];
        newImage = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_BYTE_GRAY);
        // log2(xfaktor2) ausrechnen
        i = xfaktor2;
        xf2shift = 0;
        while (i>1) {
			i /= 2;
			xf2shift++;
       }
        // log2(yfaktor2) ausrechnen
        i = yfaktor2;
        yf2shift = 0;
        while (i>1) {
            i /= 2;
            yf2shift++;
       }
       
      	for (x=0; x<image.getWidth(); x+=2)
      		for (y=0; y<image.getHeight(); y+=2) {
      			// ersma' zeilenweise
      			for (j=0; j<2; j++) {
      				p = image.getRGB(x, y+j) & 0xFF;
      				q = image.getRGB(x+1, y+j) & 0xFF;
      				sum = (3*xfaktor-1)*p + (1-xfaktor)*q;
      				incr = 2*(-p+q);
      				for (i=0; i<xfaktor2; i++) {
      					zeilen[i][j] = sum >> xf2shift;
      					sum += incr;
      				}
      			}
      			// dann spaltenweise
      			newX = xfaktor*x;
      			for (i=0; i<xfaktor2; i++) {
      				p = zeilen[i][0];
      				q = zeilen[i][1];
      				sum = (3*yfaktor-1)*p + (1-yfaktor)*q;
      				incr = 2*(-p+q);
      				newY = yfaktor*y;
      				for (j=0; j<yfaktor2; j++) {
      					val = sum >> yf2shift;
      					if (val<0)
      						val = 0;
      					else if (val>255)
      						val = 255;
      					newImage.setRGB(newX, newY, (val<<16)+(val<<8)+val);
      					sum += incr;
      					newY++;
      				}
      				newX++;
      			}
      		}
      	
        return newImage;
    }
	
    /**
     * Langsamste Variante des Supersamplings, dafr beste Qualitt.
     * Die neue Gre ergibt sich aus der von <CODE>this.image</CODE>.
     * Es wird <CODE>scaleForgetAboutAspectRatio</CODE> aufgerufen.
     * (Siehe Bemerkung zu Graubildern bei <CODE>subsampleHighQual</CODE>)
     * @author  Hari
     * @version 1 - 9.1.2000
     * @param   image
     * @returns java.awt.image.BufferedImage
     */
    private BufferedImage supersampleHighQual(BufferedImage image) {
    	int newWidth = this.image.getWidth();
    	int newHeight = this.image.getHeight();
    	return scaleForgetAboutAspectRatio(image, newWidth, newHeight);
    }
	
    /**
     * Diese Methode fhrt das (Farb-)Supersampling durch (Umkehrung des Subsamplings);
     * dabei wird ein neues <CODE>BufferedImage</CODE> zurckgeliefert, das Alte bleibt unverndert.
     * Es wird entweder <CODE>supersampleLowQual</CODE>, <CODE>supersampleMediumQual</CODE>
     * oder <CODE>supersampleHighQual</CODE> aufgerufen.
     * @author  Hari
     * @version 4 - 8.1.2000
     * @param   image 
     * @returns java.awt.image.BufferedImage
     */
    private BufferedImage supersample(BufferedImage image) {
    	int newWidth = this.image.getWidth();
    	int newHeight = this.image.getHeight();
    	if (newWidth==image.getWidth() && newHeight==image.getHeight()) {
    		BufferedImage newImage = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_BYTE_GRAY);
    		Graphics2D g2D = newImage.createGraphics();
    		g2D.drawImage(image, 0, 0, this);
    		return newImage;
    	}
        
    	if (FrameVisJPEG.vis.parameter.skalierungsQualitaet == 0)
    		return supersampleLowQual(image);
    	else if (FrameVisJPEG.vis.parameter.skalierungsQualitaet == 1)
    		return supersampleMediumQual(image);
    	else
    		return supersampleHighQual(image);
    }
	
    /**
     * Diese Methode liefert den aktuell vom Benutzer ausgewhlten 8x8-Block des Y-Teilbildes
     * in einem <CODE>Block</CODE>-Objekt zurck.
     * @author  Raimund Specht
     * @version 1-04.12.1999
     * @returns Block
     */
    public Block getMarkedBlock() {
        return getYBlock(markerX, markerY);
    }
        
    /**
     * Diese Methode liest einen 8x8-Block aus dem Y-Teilbild aus und liefert ihn als 
     * <CODE>Block</CODE>-Objekt zurck.
     * @author  Raimund Specht
     * @version 2-08.12.1999
     * @param   blockX Eine x-Koordinate der gewnschten Blockposition (Wertebereich: 0 - 192).
     * @param   blockY Eine y-Koordinate der gewnschten Blockposition (Wertebereich: 0 - 144).
     * @returns Block
     */
    public Block getYBlock(int blockX, int blockY) {
        // ggf. Y-Teilbild berechnen
        if (yImage == null) splitToYIQ();
        
        return getBlock(yImage, blockX, blockY);
    }

    /**
     * Diese Methode liest unter Bercksichtigung des Subsamplings einen 
     * 8x8-Block aus dem I-Teilbild aus und liefert ihn als 
     * <CODE>Block</CODE>-Objekt zurck.
     * @author  Raimund Specht, Hari
     * @version 2-08.12.1999
     * @param   blockX Eine x-Koordinate der gewnschten Blockposition (Wertebereich: 0 - 192).
     * @param   blockY Eine y-Koordinate der gewnschten Blockposition (Wertebereich: 0 - 144).
     * @param   subsample
     * @returns Block Der gewnschte Subsampling-Faktor.
     */
    public Block getIBlock(int blockX, int blockY, int hsubsample, int vsubsample) {
        BufferedImage subsampledImage;
        
        subsampledImage = getIImage(hsubsample, vsubsample);
        return getBlock(subsampledImage, blockX, blockY);
    }
	
    /**
     * Diese Methode liest unter Bercksichtigung des Subsamplings einen 
     * 8x8-Block aus dem Q-Teilbild aus und liefert ihn als 
     * <CODE>Block</CODE>-Objekt zurck.
     * @author  Raimund Specht, Hari
     * @version 2-08.12.1999
     * @param   blockX Eine x-Koordinate der gewnschten Blockposition (Wertebereich: 0 - 192).
     * @param   blockY Eine y-Koordinate der gewnschten Blockposition (Wertebereich: 0 - 144).
     * @param   subsample Der gewnschte Subsampling-Faktor.
     * @returns Block
     */
    public Block getQBlock(int blockX, int blockY, int hsubsample, int vsubsample) {
        BufferedImage subsampledImage;
        
        subsampledImage = getQImage(hsubsample, vsubsample);
        return getBlock(subsampledImage, blockX, blockY);
    }
    
    /**
     * Baut aus einem <CODE>Vector</CODE> von <CODE>Block</CODE> Elementen
     * einen Bildkanal. Zurckgegeben wird ein BufferedImage, das gleich
     * gro ist wie <CODE>image</CODE>.
     * Die Anzahl der Blcke im Vektor muss stimmen.
     * Das Argument <CODE>resampleRatio</CODE> gibt den Skalierungsfaktor an.
     * Daraus wird die Anzahl der auszulesenden Blcke bestimmt.
     * @author  Hari
     * @version 1 - 1.1.00
     * @param   Vector
     * @param   int resampleRatio
     * @param   qMatrix Quantisierungsmatrix
     * @returns BufferedImage
     */
    private BufferedImage imageFromBlocks(Vector bVec, int resampleRatioH, int resampleRatioV, Matrix qMatrix) {
    	BufferedImage newImage;
    	int x, y, i;
    	Block b;
    	FloatBlock fb;
    	int numXBlocks, numYBlocks;

	    numXBlocks = xBlockx(resampleRatioH);
	    numYBlocks = yBlockx(resampleRatioV);
	    newImage = new BufferedImage(image.getWidth()/resampleRatioH, image.getHeight()/resampleRatioV, BufferedImage.TYPE_BYTE_GRAY);
	    i = 0;
        for (y=0; y<numYBlocks; y++)
            for (x=0; x<numXBlocks; x++) {
	    		b = (Block)bVec.elementAt(i);
	    		b = b.getDequantisiert(qMatrix);
	    		b = b.getIDCT();
	    		insertBlockInImage(newImage, b, x*8, y*8);
	    		i++;
	    	}
	    return newImage;
    }
    
	/**
	 * Liefert den Fehler des bergebenen Bildes relativ zum Originalbild
	 * zurck.
	 * Es wird die Summe der Quadrate der pixel- und kanalweisen 
	 * Differenzen berechnet und durch die Summe der
	 * pixel- und kanalweisen Quadrate des Originalbildes geteilt.
	 * @author Hari
	 * @Version 2 - 16.1.2000
	 * returns float
	 */
	public String getDeviance() {
		int x, y;
		long err, sum;
		int c1, c2, v1, v2;
		int percentage;
		
		err = sum = 0;
		for (x=firstActCol; x<lastActCol; x++)
			for (y=firstActLine; y<lastActLine; y++) {
				c1 = image.getRGB(x, y);
				c2 = decomprRGBImage.getRGB(x, y);
				
				v1 = (c1 & 0xff0000) >> 16;
				v2 = (c2 & 0xff0000) >> 16;
				err += (v1-v2)*(v1-v2);
				sum += v1*v1;
				
				v1 = (c1 & 0x00ff00) >> 8;
				v2 = (c2 & 0x00ff00) >> 8;
				err += (v1-v2)*(v1-v2);
				sum += v1*v1;
				
				v1 = c1 & 0x0000ff;
				v2 = c2 & 0x0000ff;
				err += (v1-v2)*(v1-v2);
				sum += v1*v1;
			}
        
        if (sum != 0)
            percentage = (int)(1000*err/sum);
        else
            percentage = 0;
        return new String(new Float(((float)percentage)/10).toString()+"%");
	}
	
    /**
     * Diese Methode berechnet die Unterschiede zwischen dem Originalbild und dem JPEG-komprimierten Bild
     * und stellt diese in einem <CODE>BufferedImage</CODE> dar.
     * @author  Raimund Specht
     * @version 1-26.12.1999
     * @param   parameter Die zu verwendenden JPEG-Parameter.
     * @returns java.awt.image.BufferedImage
     */
    public BufferedImage getDifferenceImage(JPEGParameter parameter) {
        int x, y, colorOrig, colorJPEG;
        int diffR, diffG, diffB;
        BufferedImage jpeg;

        if (!parameter.equals(differenceImageParameter)) {
            differenceImageParameter = (JPEGParameter)parameter.clone();
            jpeg = getDecompressedJPEGImage(parameter);
            diffImage = new BufferedImage(192, 144, BufferedImage.TYPE_INT_RGB);
            for (x=0; x<192; x++)
                for (y=0; y<144; y++) {
                    colorOrig = image.getRGB(x, y);
                    colorJPEG = jpeg.getRGB(x, y);
                    diffR = 2*Math.abs(((colorOrig & 0xFF0000) >> 16) - ((colorJPEG & 0xFF0000) >> 16));
                    if (diffR>255) diffR=255;
                    diffG = 2*Math.abs(((colorOrig & 0x00FF00) >> 8) - ((colorJPEG & 0x00FF00) >> 8));
                    if (diffG>255) diffG=255;
                    diffB = 2*Math.abs((colorOrig & 0x0000FF) - (colorJPEG & 0x0000FF));
                    if (diffB>255) diffB=255;
                    diffImage.setRGB(x, y, (diffR << 16) + (diffG << 8) + diffB);
                }
        }
        
        return diffImage;
    }

    /**
     * Diese Methode berechnet ein komprimiertes und wieder dekomprimiertes
     * JPEG Bild.
     * Die Schritte der JPEG Kompression werden bis zum Ende (inkl.
     * Huffman) durchgefhrt, damit wir auch die Bildgre kriegen.
     * Diese wird - sehr unschn - ins <CODE>FrameVisJPEG</CODE> Objekt
     * geschrieben.
     * @author  Hari
     * @version 5 - 11.1.2000
     * @param   parameter Die zu verwendenden JPEG-Parameter.
     * @returns java.awt.image.BufferedImage
     * @see     JPEGParameter
     */
    public BufferedImage getDecompressedJPEGImage(JPEGParameter parameter) {
        BufferedImage decomprYImage, decomprIImage, decomprQImage;
        
        if (!parameter.equals(jpegImageParameter)) {
            jpegImageParameter = (JPEGParameter)parameter.clone();
            // Bild transformieren bis zur Quantisierung, wir kriegen die quantisierten Blcke
            quantBloeckeErzeugen(parameter);
            // Bild codieren, wir kriegen die Symbolfolgen
            symboleErzeugen(parameter);
            // die Symbolfolge codieren, wir kriegen die Lnge des JPEG Bildes in codedSize
            symboleCodieren(parameter);
            FrameVisJPEG.gesamtKompression = (int)(100.0*codedSize/(192*144*3)+0.5);
        
            // dekomprimiertes Bild ausrechnen
            decomprYImage = imageFromBlocks(quantYBloecke, 1, 1, parameter.qMatrixLum);
            decomprIImage = imageFromBlocks(quantIBloecke, parameter.hSubsample, parameter.vSubsample, parameter.qMatrixChrom);
            decomprQImage = imageFromBlocks(quantQBloecke, parameter.hSubsample, parameter.vSubsample, parameter.qMatrixChrom);
            decomprRGBImage = yiqToRGB(decomprYImage, decomprIImage, decomprQImage, parameter.hSubsample, parameter.vSubsample);
        } 

        return decomprRGBImage;
    }
    
    /**
     * Diese Methode liest einen 8x8-Block aus dem bergebenen BufferedImage aus und liefert ihn als 
     * <CODE>Block</CODE>-Objekt zurck.
     * @author  Raimund Specht, Hari
     * @version 3-1.1.00
     * @param   image  Das BufferedImage, aus dem der Block ausgelesen werden soll
     * @param   blockX Die x-Position des auszulesenden Blocks
     * @param   blockY Die y-Position des auszulesenden Blocks
     * @returns Block
     */
    public static Block getBlock(BufferedImage image, int blockX, int blockY) {
        Block b;
        int x, y, i, j, xx, yy;
        
        b = new Block();
        i = 0;
        for (x = blockX; x < blockX+8; x++) {
            j=0;
            if (x<image.getWidth()) xx=x; else xx=image.getWidth()-1;	// Damit wird der Rand des Bilds nach auen quasi verlngert, damit Bereichsberschreitungen nicht zu Fehlern fhren!
            if (x<0) xx=0;
            for (y = blockY; y < blockY+8; y++) {
            	if (y<image.getHeight()) yy=y; else yy=image.getHeight()-1;  // Dito
            	if (y<0) yy=0;
                b.matrix[i][j] = image.getRGB(xx, yy) & 0xFF;  
                j++;
            }
            i++;
        }
        return b;
    } // getBlock()
	
    /**
     * Diese Methode baut ein <CODE>Block</CODE>-Objekt in ein BufferedImage ein. Die Methode entspricht somit etwa
     * der Umkehrung von <CODE>getBlock()</CODE>. 
     * @author  Raimund Specht
     * @version 2-14.12.1999
     * @param   img    Das BufferedImage, in das der Block eingefgt werden soll (wird nicht kopiert, sondern 
     *                 direkt verndert)
     * @param   block  Der Block, der in das Bild eingefgt werden soll
     * @param   blockX Die x-Position, an der der Block in das Bild eingefgt werden soll
     * @param   blockY Die y-Position, an der der Block in das Bild eingefgt werden soll
     */
    private void insertBlockInImage(BufferedImage img, Block block, int blockX, int blockY) {
        int i, j, x, y, col;
        
        i = 0;
        for (x = blockX; x < blockX+8; x++) {
            j=0;
            for (y = blockY; y < blockY+8; y++) {
                col = (int)block.matrix[i][j];
                
                // prfen, ob "col" im zulssigen Bereich liegt, sonst meldet Color() einen Fehler
                if (col < 0) col = 0;
                if (col > 255) col = 255;
                
                if ((x>=0)&&(y>=0)&&(x<img.getWidth())&&(y<img.getHeight()))		// nur setzen, wenn noch im Bildbereich
	                img.setRGB(x, y, (new Color(col, col, col)).getRGB());
                j++;
            }
            i++;
        }
    } // insertBlockInImage()
    
    /**
     * Liefert ein <CODE>Vector</CODE> Objekt, das Strings fr die
     * RLE Visualisierung enthlt. Die Strings werden aus dem markierten
     * Block erzeugt.
     * Der erste String enthlt die Werte vor der RLE Kodierung.
     * Die folgenden Strings sind immer abwechselnd ein String mit
     * AC Werten / ein String mit Anzahl Nullen usw.
     * @author Hari
     * @author Raimund Specht
     * @version 2 - 23.02.2000
     * @returns Vector
     */
    public Vector getRLEStrings() {
    	Vector vec;
    	String s;
    	int i, run;
    	Block block;
    BlockVektor zickzack;
    	
    	vec = new Vector();
    	block = getMarkedBlock().getDCT().getQuantisiert(FrameVisJPEG.vis.parameter.qMatrixLum);
    zickzack = block.getZigZagScan();
    	s = new String("");
    	for (i=1; i<64; i++) {
    		s = s.concat(Integer.toString(zickzack.werte[i]));
    		s = s.concat(" ");
    	}
    	vec.addElement(s);
    	i = 1;
    	while (i<64) {
    		s = new String("");
    		while (i<64 && zickzack.werte[i]!=0) {
    			s = s.concat(Integer.toString(zickzack.werte[i]));
    			s = s.concat(" ");
    			i++;
    		}
    		vec.addElement(s);
    		run = 0;
    		while (i<64 && zickzack.werte[i]==0) {
    			i++;
    			run++;
    		}
    		vec.addElement(Integer.toString(run));
    	}
    	return vec;
    }

    /**
     * Diese Methode berfhrt das komplette Bild in eine Folge aus
     * quantisierten Blcken. Diese stehen danach in <CODE>quantYBloecke</CODE>,
     * <CODE>quantIBloecke</CODE> und <CODE>quantQBloecke</CODE>.
     * Die Methode sorgt schommal dafr, dass die Anzahl der Y-Blockzeilen
     * und -Spalten immer ein  Vielfaches der U- und V-Blockzeilen/Spalten ist,
     * damit spter ganze MCUs erzeugt werden knnen.
     * Wir merken uns in <CODE>quantBloeckeErzeugenParams</CODE> die Parameter
     * beim letzten Aufruf, damit wir nur neu berechnen wenn ntig.
     * @author  Hari
     * @version 2 - 4.2.00
     * @param JPEGParameter
     */
    private void quantBloeckeErzeugen(JPEGParameter parameter) {
        Block block;
        FloatBlock fBlock;
        Matrix quantMatrixLum, quantMatrixChrom;
        int x, y;
        int lumWidth, lumHeight, chromWidth, chromHeight;
        BufferedImage iImage, qImage;

        if (parameter.equals(quantBloeckeErzeugenParams))
            return;
        quantBloeckeErzeugenParams = (JPEGParameter)parameter.clone();

        lumWidth = xBlockx(1);
        lumHeight = yBlockx(1);
        chromWidth = xBlockx(parameter.hSubsample);
        chromHeight = yBlockx(parameter.vSubsample);
        // jetzt noch sicher machen, dass die Zahl der Block Zeilen
        // und Spalten immer ein Vielfaches voneinander ist.
        // also y Blcke dazu fgen, bis (auch am Rand) ganze MCUs entstehen
        while (lumWidth%chromWidth>0)
            lumWidth++;
        while (lumHeight%chromHeight>0)
            lumHeight++;
            
        dcLumWerte = new int[lumWidth*lumHeight];
        // Y-Kanal
        quantMatrixLum = parameter.qMatrixLum;
        quantYBloecke = new Vector();
        for (y=0; y<lumHeight; y++)
            for (x=0; x<lumWidth; x++) {
                fBlock = getYBlock(x*8, y*8).getDCT();
                block = fBlock.getQuantisiert(quantMatrixLum);
                dcLumWerte[y*lumWidth+x] = (int)block.matrix[0][0];
                quantYBloecke.addElement(block);
            }
        // Cr-Kanal
        quantMatrixChrom = parameter.qMatrixChrom;
        dcChromWerte = new int[2*chromWidth*chromHeight];
        quantIBloecke = new Vector();
        iImage = getIImage(parameter.hSubsample, parameter.vSubsample);
        for (y=0; y<chromHeight; y++)
            for (x=0; x<chromWidth; x++) {
                fBlock = JPEGImage.getBlock(iImage, x*8, y*8).getDCT();
                block = fBlock.getQuantisiert(quantMatrixChrom);
                dcChromWerte[y*chromWidth+x] = (int)block.matrix[0][0];
                quantIBloecke.addElement(block);
            }
    
        // Cb-Kanal
        quantQBloecke = new Vector();
        qImage = getQImage(parameter.hSubsample, parameter.vSubsample);
        for (y=0; y<chromHeight; y++)
            for (x=0; x<chromWidth; x++) {
                fBlock = JPEGImage.getBlock(qImage, x*8, y*8).getDCT();
                block = fBlock.getQuantisiert(quantMatrixChrom);
                dcChromWerte[chromWidth*chromHeight + y*chromWidth+x] = (int)block.matrix[0][0];
                quantQBloecke.addElement(block);
            }
    }
    
    /**
     * Diese Methode berfhrt die 3 Folgen der quantisierten Blcke
     * in eine Folge aus  AC- und DC Symbolen. Diese stehen danach in
     * <CODE>acLumSymbole</CODE>, <CODE>acChromSymbole</CODE>,
     * <CODE>dcLumSymbole</CODE> und <CODE>dcChromSymbole</CODE>.
     * Wir merken uns in <CODE>symboleErzeugenParams</CODE> die Parameter
     * beim letzten Aufruf, damit wir nur neu berechnen wenn ntig.
     * Es wird das int array <CODE>deMCUOrder</CODE> erzeugt, mit dem man
     * Block-Indizes nach der MCU Block Umsortierung
     * in zeilenweise gescannte Block Indizes umrechnen kann. Nicht wahr.
     * @author  Hari
     * @version 3 - 4.2.00
     * @param JPEGParameter
     */
    private void symboleErzeugen(JPEGParameter parameter) {
        Block block;
        BlockVektor zigVektor;
        int xSizeFactor, ySizeFactor;
        int maxSampFactorH, maxSampFactorV;
        int currentmcuCol, currentmcuLine;
        int cols;
        int blockIdx;
        int mcuLines, mcuCols;
	    
        if (parameter.equals(symboleErzeugenParams))
            return;
        symboleErzeugenParams = (JPEGParameter)parameter.clone();
        
        maxSampFactorH = getMaxSampFactorHoriz(parameter);
        maxSampFactorV = getMaxSampFactorVert(parameter);
        mcuLines = yBlockx(maxSampFactorV);
        mcuCols = xBlockx(maxSampFactorH);
        
        // Y-Kanal
        yBlockFolge = new SymbBlockFolge();
        cols = xBlockx(1);
        xSizeFactor = maxSampFactorH;
        ySizeFactor = maxSampFactorV;
        deMCUOrder = new int[mcuLines*mcuCols*xSizeFactor*ySizeFactor];
        int linearBlockIdx = 0;
        for (currentmcuLine=0; currentmcuLine<mcuLines; currentmcuLine++)
            for (currentmcuCol=0; currentmcuCol<mcuCols; currentmcuCol++) {
                for (int y=0; y<ySizeFactor; y++)
                    for (int x=0; x<xSizeFactor; x++) {
                        blockIdx = (currentmcuLine*ySizeFactor+y)*cols + currentmcuCol*xSizeFactor+x;
                        deMCUOrder[blockIdx] = linearBlockIdx;
                        linearBlockIdx++;
                        block = (Block)quantYBloecke.elementAt(blockIdx);
                        zigVektor = block.getZigZagScan();
                        yBlockFolge.addElement(zigVektor.getRLECompressed());
                    }
            }

        xSizeFactor = maxSampFactorH/parameter.hSubsample;
        ySizeFactor = maxSampFactorV/parameter.vSubsample;
        cols = xBlockx(parameter.hSubsample);

	    // Cr-Kanal
        crBlockFolge = new SymbBlockFolge();
        for (currentmcuLine=0; currentmcuLine<mcuLines; currentmcuLine++)
            for (currentmcuCol=0; currentmcuCol<mcuCols; currentmcuCol++)
                for (int y=0; y<ySizeFactor; y++)
                    for (int x=0; x<xSizeFactor; x++) {
                        blockIdx = (currentmcuLine*ySizeFactor+y)*cols + currentmcuCol*xSizeFactor+x;
                        block = (Block)quantIBloecke.elementAt(blockIdx);
                        zigVektor = block.getZigZagScan();
                        crBlockFolge.addElement(zigVektor.getRLECompressed());
                    }
	
        // Cb-Kanal
        cbBlockFolge = new SymbBlockFolge();
        for (currentmcuLine=0; currentmcuLine<mcuLines; currentmcuLine++)
            for (currentmcuCol=0; currentmcuCol<mcuCols; currentmcuCol++)
                for (int y=0; y<ySizeFactor; y++)
                    for (int x=0; x<xSizeFactor; x++) {
                        blockIdx = (currentmcuLine*ySizeFactor+y)*cols + currentmcuCol*xSizeFactor+x;
                        block = (Block)quantQBloecke.elementAt(blockIdx);
                        zigVektor = block.getZigZagScan();
                        cbBlockFolge.addElement(zigVektor.getRLECompressed());
                    }
    }
    
    /**
     * Liefert den maximalen horizontalen Subsampling Faktor unter den Faktoren
     * der 3 Kanle zurck. Die Faktoren stehen im <CODE>JPEGParameter</CODE>
     * Objekt, das der Methode bergeben wird
     * @author  Hari
     * @version 1 - 4.2.00
     * @param JPEGParameter
     * @returns int
     */
    private int getMaxSampFactorHoriz(JPEGParameter parameter) {
        if (parameter.hSubsample>1)
            return parameter.hSubsample;
        else
            return 1;
    }
    
    /**
     * Liefert den maximalen vertikalen Subsampling Faktor unter den Faktoren
     * der 3 Kanle zurck. Die Faktoren stehen im <CODE>JPEGParameter</CODE>
     * Objekt, das der Methode bergeben wird
     * @author  Hari
     * @version 1 - 4.2.00
     * @param JPEGParameter
     * @returns int
     */
    private int getMaxSampFactorVert(JPEGParameter parameter) {
        if (parameter.vSubsample>1)
            return parameter.vSubsample;
        else
            return 1;
    }
    
    /**
     * Schreibt die JPEG (JFIF) Headers in den <CODE>HuffmanWriter</CODE>
     * Die drei <CODE>ints</CODE>, die bergeben werden, sind lediglich
     * die vier Grenfaktoren. Man erhlt sie, indem man die horiz/vert
     * maximalen Sampling Faktoren durch den h/v Subsampling Faktor fr die
     * Luminanz/Chrominanz Kanle teilt (siehe <CODE>doHuffmanCoding</CODE>)
     * @author  Hari
     * @version 1 - 4.2.00
     * @param HuffmanWriter
     * @param JPEGParameter
     * @param int
     * @param int
     * @param int
     * @param int
     * @returns int
     */
    private void writeHeaders(HuffmanWriter hWriter, JPEGParameter parameter, int lumSizeFactorH, int lumSizeFactorV, int chromSizeFactorH, int chromSizeFactorV) {
        int length;
        
        // SOI
        byte[] SOI = {(byte) 0xFF, (byte) 0xD8};
        hWriter.writeBytes(SOI);
        
        // JFIF header
        byte JFIF[] = new byte[18];
        JFIF[0] = (byte) 0xff;
        JFIF[1] = (byte) 0xe0;
        JFIF[2] = (byte) 0x00;
        JFIF[3] = (byte) 0x10;
        JFIF[4] = (byte) 0x4a;
        JFIF[5] = (byte) 0x46;
        JFIF[6] = (byte) 0x49;
        JFIF[7] = (byte) 0x46;
        JFIF[8] = (byte) 0x00;
        JFIF[9] = (byte) 0x01;
        JFIF[10] = (byte) 0x00;
        JFIF[11] = (byte) 0x00;
        JFIF[12] = (byte) 0x00;
        JFIF[13] = (byte) 0x01;
        JFIF[14] = (byte) 0x00;
        JFIF[15] = (byte) 0x01;
        JFIF[16] = (byte) 0x00;
        JFIF[17] = (byte) 0x00;
        hWriter.writeBytes(JFIF);

        // Comment Header
        String comment = new String("konkret! ich schwr!");
        length = comment.length() + 2;
        byte COM[] = new byte[4];
        COM[0] = (byte) 0xFF;
        COM[1] = (byte) 0xFE;
        COM[2] = (byte) ((length >> 8) & 0xFF);
        COM[3] = (byte) (length & 0xFF);
        hWriter.writeBytes(COM);
        hWriter.writeBytes(comment.getBytes());

        // DQT header
        byte DQT[] = new byte[134];
        Matrix qMatrices[] = new Matrix[2];
        DQT[0] = (byte) 0xFF;
        DQT[1] = (byte) 0xDB;
        DQT[2] = (byte) 0x00;
        DQT[3] = (byte) 0x84;
        length = 4;
        qMatrices[0] = parameter.qMatrixLum.getZigZagOrdered();
        qMatrices[1] = parameter.qMatrixChrom.getZigZagOrdered();
        for (int i=0; i<2; i++) {
            DQT[length] = (byte)((0<<4) + i);  // qt info
            length++;
            for (int y=0; y<8; y++)
                for (int x=0; x<8; x++) {
                    DQT[length] = (byte)(qMatrices[i].matrix[x][y]);
                    length++;
                }
        }
        hWriter.writeBytes(DQT);
        
        // Start of Frame 0 Header
        byte SOF[] = new byte[19];
        SOF[0] = (byte) 0xFF;
        SOF[1] = (byte) 0xC0;
        SOF[2] = (byte) 0x00;
        SOF[3] = (byte) 17;
        SOF[4] = (byte) 8;  // precision
        SOF[5] = (byte) ((144 >> 8) & 0xFF);  // img height lo byte
        SOF[6] = (byte) ((144) & 0xFF);  // img height hi byte
        SOF[7] = (byte) ((192 >> 8) & 0xFF);  // img width lo byte
        SOF[8] = (byte) ((192) & 0xFF);  // img width lo byte
        SOF[9] = (byte) 3;  // number of components
        SOF[10] = (byte) 1;  // component id
        SOF[11] = (byte) ((lumSizeFactorH << 4) + lumSizeFactorV);  // hor/vert sampling factor
        SOF[12] = (byte) 0;  // quantization table number
        SOF[13] = (byte) 2;  // component id
        SOF[14] = (byte) ((chromSizeFactorH << 4) + chromSizeFactorV);  // hor/vert sampling factor
        SOF[15] = (byte) 1;  // quantization table number
        SOF[16] = (byte) 3;  // component id
        SOF[17] = (byte) ((chromSizeFactorH << 4) + chromSizeFactorV);  // hor/vert sampling factor
        SOF[18] = (byte) 1;  // quantization table number
        hWriter.writeBytes(SOF);

        // DHT Header
        OurHuffmanTable hTables[] = new OurHuffmanTable[4];
        byte segBuffer[] = new byte[1096];  // max. gre fr segment
        length = 4;
        segBuffer[0] = (byte) 0xFF;  // DHT marker
        segBuffer[1] = (byte) 0xC4;
        hTables[0] = dcLumHuffmanTable;
        hTables[1] = acLumHuffmanTable;
        hTables[2] = dcChromHuffmanTable;
        hTables[3] = acChromHuffmanTable;
        for (int i=0; i<4; i++) {
            byte htInfo;
            if (i%2==0)  // dc table
                htInfo = (byte)(i/2);
            else
                htInfo = (byte)(i/2+16);
            segBuffer[length] = htInfo;
            length++;
            for (int j=1; j<=16; j++) {
                segBuffer[length] = (byte)hTables[i].codedLengths[j];
                length++;
            }
            for (int j=0; j<hTables[i].codedSymbols.length; j++) {
                segBuffer[length] = (byte)hTables[i].codedSymbols[j];
                length++;
            }
        }
        segBuffer[2] = (byte)((length-2) >> 8);
        segBuffer[3] = (byte)((length-2) & 0xFF);
        hWriter.writeBytes(segBuffer, length);
        
        // Start of Scan Header
        byte SOS[] = new byte[14];
        SOS[0] = (byte) 0xFF;
        SOS[1] = (byte) 0xDA;
        SOS[2] = (byte) 0x00;
        SOS[3] = (byte) 12;
        SOS[4] = (byte) 3;  // number of components
        SOS[5] = (byte) 1;  // component id
        SOS[6] = (byte) ((0 << 4) + 0);  // dc/ac table number
        SOS[7] = (byte) 2;  // component id
        SOS[8] = (byte) ((1 << 4) + 1);  // dc/ac table number
        SOS[9] = (byte) 3;  // component id
        SOS[10] = (byte) ((1 << 4) + 1);  // dc/ac table number
        SOS[11] = (byte) 0;  // Ss
        SOS[12] = (byte) 63;  // Se
        SOS[13] = (byte) ((0 << 4) + 0);  // Ah/Al
        hWriter.writeBytes(SOS);
    }

    /**
     * Schreibt die Bild-Ende Markierung in den <CODE>HuffmanWriter</CODE>
     * @author  Hari
     * @version 1 - 4.2.00
     * @param HuffmanWriter
     */
     private void writeEOI(HuffmanWriter hWriter) {
        byte[] EOI = {(byte) 0xFF, (byte) 0xD9};
        hWriter.writeBytes(EOI);
    }
    
    /**
     * Fhrt die Huffman Kodierung auf den kodierten Symbolen durch.
     * Wir erzeugen einen <CODE>HuffmanWriter</CODE> und kodieren dann
     * jeden Block. Falls das Prog als App luft, wird anschlieend
     * noch das JPG File auf Pladde geschrieben.
     * Auch die Lnge <CODE>lengthHuffmanCoded</CODE> des JPEG Streams
     * (ohne Headers) und die Lnge des fertig kodierten Bildes (mit
     * den Headern) in <CODE>codedSize</CODE> wird hier gesetzt.
     * @author  Hari
     * @version 2 - 4.2.00
     * @parameter JPEGParameter
     */
    private void doHuffmanCoding(JPEGParameter parameter) {
        Histogramm histo;

        codedSize = 0;
        // Huffman Tabelle fr Y Kanal
        if (parameter.compressionType == JPEGParameter.HUFFMAN_STANDARD) {
            dcLumHuffmanTable = new OurHuffmanTable(JPEGHuffmanTable.StdDCLuminance);
            acLumHuffmanTable = new OurHuffmanTable(JPEGHuffmanTable.StdACLuminance);
        }    
        else {
            histo = yBlockFolge.getDCFreqHisto();
            dcLumHuffmanTable = histo.buildHuffmanTable();
            histo = yBlockFolge.getACFreqHisto();
            acLumHuffmanTable = histo.buildHuffmanTable();
        }
        
        // Huffman Tabelle fr Cr und Cb Kanal
        if (parameter.compressionType == JPEGParameter.HUFFMAN_STANDARD) {
            dcChromHuffmanTable = new OurHuffmanTable(JPEGHuffmanTable.StdDCChrominance);
            acChromHuffmanTable = new OurHuffmanTable(JPEGHuffmanTable.StdACChrominance);
        }    
        else {
            histo = crBlockFolge.getDCFreqHisto();
            histo.addHisto(cbBlockFolge.getDCFreqHisto());
            dcChromHuffmanTable = histo.buildHuffmanTable();
            histo = crBlockFolge.getACFreqHisto();
            histo.addHisto(cbBlockFolge.getACFreqHisto());
            acChromHuffmanTable = histo.buildHuffmanTable();
        }
        
        // Okay, jetzt kommt die Kodierung des Bildes
        int sizeOfHeaders;
        int maxSampFactorH = getMaxSampFactorHoriz(parameter);
        int maxSampFactorV = getMaxSampFactorVert(parameter);
        int lumSizeFactorH = maxSampFactorH;
        int lumSizeFactorV = maxSampFactorV;
        int chromSizeFactorH = maxSampFactorH/parameter.hSubsample;
        int chromSizeFactorV = maxSampFactorV/parameter.vSubsample;
        
        hWriter = new HuffmanWriter();
        writeHeaders(hWriter, parameter, lumSizeFactorH, lumSizeFactorV, chromSizeFactorH, chromSizeFactorV);
        sizeOfHeaders = hWriter.getBitsWritten();

        int yBlockIdx = 0;
        int crBlockIdx = 0;
        int cbBlockIdx = 0;
        for (int mcuy=0; mcuy<yBlockx(maxSampFactorV); mcuy++)
            for (int mcux=0; mcux<xBlockx(maxSampFactorH); mcux++) {
                for (int i=0; i<lumSizeFactorH*lumSizeFactorV; i++) {
                    yBlockFolge.huffmanCodeBlock(yBlockIdx, dcLumHuffmanTable, acLumHuffmanTable, hWriter);
                    yBlockIdx++;
                }
                for (int i=0; i<chromSizeFactorH*chromSizeFactorV; i++) {
                    crBlockFolge.huffmanCodeBlock(crBlockIdx, dcChromHuffmanTable, acChromHuffmanTable, hWriter);
                    crBlockIdx++;
                }
                for (int i=0; i<chromSizeFactorH*chromSizeFactorV; i++) {
                    cbBlockFolge.huffmanCodeBlock(cbBlockIdx, dcChromHuffmanTable, acChromHuffmanTable, hWriter);
                    cbBlockIdx++;
                }
            }

        hWriter.flush();
        lengthHuffmanCoded = hWriter.getBitsWritten() - sizeOfHeaders;
        writeEOI(hWriter);

        codedSize = hWriter.getBitsWritten();
        codedSize /= 8;  // in Bytes umrechnen

    }
       
    /**
     * Speichert die JPG Datei. berschreibt hemmungslos, falls schon vorhanden
     * @author  Hari
     * @version 1 - 7.2.00
     * @parameter String filename
     */
    public boolean writeToFile(String filename) {
        return hWriter.writeToFile(filename);
    }
     
    /**
     * Macht eine DPCM, VLI und Huffman Codierung auf den Symbolfolgen,
     * die zuvor durch <CODE>symboleErzeugen</CODE> generiert wurden.
     * Es werden also die fehlenden Schritte der JPEG Komprimierung
     * vollendet.
     * Wir merken uns in <CODE>symboleCodierenParams</CODE> die Parameter
     * beim letzten Aufruf, damit wir nur neu berechnen wenn ntig.
     * @author  Hari
     * @version 2 - 4.2.00
     * @parameter JPEGParameter
     */
    private void symboleCodieren(JPEGParameter parameter) {
        if (parameter.equals(symboleCodierenParams))
            return;
        symboleCodierenParams = (JPEGParameter)parameter.clone();

        // DPCM auf allen 3 Kanlen
        yBlockFolge.doDPCMCoding();
        crBlockFolge.doDPCMCoding();
        cbBlockFolge.doDPCMCoding();
        
         // VLI auf allen 3 Kanlen
        yBlockFolge.doVLICoding();
        crBlockFolge.doVLICoding();
        cbBlockFolge.doVLICoding();
        
        doHuffmanCoding(parameter);
    }
    
    /**
     * Gibt eine reduzierte Anzahl von DC Werten zurck (fr die Darstellung
     * im DPCM/RLE Panel).
     * Die Anzahl steht in <CODE>laengeReduziert</CODE> (lokale Konstante).
     * Es werden die DC Werte der Blcke links und rechts vom aktuellen
     * Block genommen.
     * Es werden die Blcke von der letzten Kodierung des Bildes, d.h. vom
     * letzten Aufruf von <CODE>symboleErzeugen</CODE> benutzt.
     * @author Hari
     * @author Raimund Specht
     * @version 3 - 23.02.2000
     * @returns int[]
     */
    private int[] getDCWerteReduziert() {
        final int laengeReduziert = 26;
        int[] dcWerteReduziert = new int[laengeReduziert];
        int i;
        boolean linkerRandErreicht = false;

        indexAktuellerBlock = 1+laengeReduziert/2;
        int reduzStart = (markerY/8)*xBlockx(1) + markerX/8 - (laengeReduziert)/2;
        if (reduzStart <= 0) {
            indexAktuellerBlock += reduzStart;
            reduzStart = 0;
            linkerRandErreicht = true;
        }
        if (reduzStart > dcLumWerte.length-laengeReduziert+1) {
            indexAktuellerBlock += reduzStart-(dcLumWerte.length-laengeReduziert+1);
            reduzStart = dcLumWerte.length-laengeReduziert+1;
        }

        if (linkerRandErreicht)
            dcWerteReduziert[0] = 0;
        else
            dcWerteReduziert[0] = dcLumWerte[reduzStart];
        for (i=1; i<laengeReduziert; i++)
            dcWerteReduziert[i] = dcLumWerte[i+reduzStart-1];
            
        return dcWerteReduziert;
    }

    /**
     * Malt die DC Werte als Balkendiagramm. Auerdem werden ins Diagramm die
     * Differenzen aufeinanderfolgender Werte eingezeichnet.
     * Es werden die DC Werte von der letzten Kodierung des Bildes, d.h. vom
     * letzten Aufruf von <CODE>symboleErzeugen</CODE> benutzt.
     * @author Hari
     * @version 1 - 29.12.99
     * @param int width
     * @param int height
     * @param Color red (fr die Amplituden)
     * @param Color green (fr die Amplitudendifferenzen)
     * @returns BufferedImage
     */
    public BufferedImage paintDPCMBar(int width, int height, Color green, Color red, Color yellow) {
        int i, maxValue;
        float faktor;
        BufferedImage image;
        Graphics2D g2D;
        int werte[];
        int max, min;
        int x, prevY, thisY;
        int balkenBreite, luecke;
        int nullLinie;
        int randUnten = 20;
        
        image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        g2D = image.createGraphics();
        g2D.setColor(GUIParameter.INACTIVE_BACKGROUND);
        g2D.fillRect(0, 0, width, height);
        
        werte = getDCWerteReduziert();
        max = -2047;
        min = 2047;
        // Maximum und Minimum ber alle Werte und alle Differenzen bilden
        for (i=0; i<werte.length; i++) {
            if (werte[i] > max)
                max = werte[i];
			if (i>0 && werte[i]-werte[i-1]>max)
				max = werte[i]-werte[i-1];
            if (werte[i] < min)
                min = werte[i];
			if (i>0 && werte[i]-werte[i-1]<min)
				min = werte[i]-werte[i-1];
        }
        if (min > 0)  // Nulllinie soll auf jeden Fall auf dem Diagramm sein
        	min = 0;

	    faktor = (float)(height-5-randUnten-5) / (max-min);
			
		nullLinie = height-randUnten-5 - (int)((-min)*faktor);
		g2D.setColor(Color.black);
		g2D.drawLine(0, nullLinie, width-1, nullLinie);
		
        balkenBreite = width / (2*(werte.length-1));
        luecke = width / (2*(werte.length-1));
        x = 0;
        prevY = werte[0];
		for (i=1; i<werte.length; i++) {
			Polygon poly;
			
			thisY = werte[i];
			poly=new Polygon();
			poly.addPoint(x, nullLinie);
			poly.addPoint(x+balkenBreite, nullLinie);
			poly.addPoint(x+balkenBreite, height-randUnten-5 - (int)((thisY-min)*faktor));
			poly.addPoint(x, height-randUnten-5 - (int)((thisY-min)*faktor));
              g2D.setColor(red);
			g2D.fillPolygon(poly);
              if (i == indexAktuellerBlock) 
                  g2D.setColor(yellow);
              else
			    g2D.setColor(Color.black);
			g2D.drawPolygon(poly);
			
			poly=new Polygon();
			poly.addPoint(x+balkenBreite/4+1, nullLinie);
			poly.addPoint(x+balkenBreite*3/4, nullLinie);
			poly.addPoint(x+balkenBreite*3/4, height-randUnten-5 - (int)((thisY-prevY-min)*faktor));
			poly.addPoint(x+balkenBreite/4+1, height-randUnten-5 - (int)((thisY-prevY-min)*faktor));
			g2D.setColor(green);
			g2D.fillPolygon(poly);
			g2D.setColor(Color.black);
			g2D.drawPolygon(poly);
			
			prevY = thisY;
			x += balkenBreite+luecke;
        }

        // Erklrungstext zeichnen
        FontMetrics fm = g2D.getFontMetrics();
        String str;
        int strHeight = fm.getHeight();
        
        g2D.setColor(red);
        x = 5;
        g2D.fillRect(x, height-5-strHeight/2-2, balkenBreite, strHeight/2);
        x += balkenBreite+5;
        g2D.setColor(Color.black);
        str = new String("unkodierte DC-Werte");
        g2D.drawString(str, x, height-5);
        x += fm.stringWidth(str) + 7;
        
        g2D.setColor(green);
        g2D.fillRect(x, height-5-strHeight/2-2, balkenBreite, strHeight/2);
        x += balkenBreite+5;
        g2D.setColor(Color.black);
        str = new String("DPCM-Kodierung");
        g2D.drawString(str, x, height-5);
        x += fm.stringWidth(str) + 7;

        g2D.setColor(red);
        g2D.fillRect(x, height-5-strHeight/2-2, balkenBreite, strHeight/2);
        g2D.setColor(yellow);
        g2D.drawRect(x, height-5-strHeight/2-2, balkenBreite, strHeight/2);
        x += balkenBreite+5;
        g2D.setColor(Color.black);
        str = new String("aktueller Block");
        g2D.drawString(str, x, height-5);
        
        return image;
    }

    /**
     * Malt ein Histogramm ber die Lngen der <CODE>RLESymbol2</CODE>'s
     * der Symbolfolgen des Bildes. Wird nach der VLI Kodierung aller 4
     * Teilfolgen aufgerufen.
     * @author Hari
     * @version 2 - 2.1.00
     * @param int width
     * @param int height
     * @param Color
     * @returns BufferedImage
     */
    public BufferedImage paintVLIBar(int width, int height, Color c) {
        Histogramm histo, yHisto, crHisto, cbHisto;
        yHisto = yBlockFolge.getVLIHistogramm();
        crHisto = crBlockFolge.getVLIHistogramm();
        cbHisto = cbBlockFolge.getVLIHistogramm();
        
        histo = yHisto;
        histo.addHisto(crHisto);
        histo.addHisto(cbHisto);
        return histo.bitlaengenHistogramm(width, height, c);
    }

    /**
     * Codiert das Bild neu und liefert einen Vector zurck, der die
     * VLI Statistik und den VLI Kompressionsfaktor enthlt.
     * Wird von <CODE>PanelDPCMRLE</CODE> benutzt.
     * @author  Hari
     * @version 1 - 15.1.2000
     * @returns Vector
     * @see     PanelDPCMRLE
     */
	public Vector getDPCMVLIStats() {
        int sizeBefore, sizeAfter;
        int compressionRatio;
        Vector vec;
        
	    // ganzes Bild neu berechnen und codieren, um die 3 SymbBlockFolgen zu erhalten
        quantBloeckeErzeugen(FrameVisJPEG.vis.parameter);
	    symboleErzeugen(FrameVisJPEG.vis.parameter);
        symboleCodieren(FrameVisJPEG.vis.parameter);
        
//        sizeBefore = 192*144*3*8;  // *8 damit in bits
// Ich hab das hier noch in letzter Minute verndert. Er geht jetzt bei der
// "Gre vorher" nicht vom geladenen Bild aus, sondern bercksichtigt die
// Datenreduktion beim Farbsubsampling. -hArI
        float uvFaktor = (float)(1+2.0/(FrameVisJPEG.vis.parameter.hSubsample*FrameVisJPEG.vis.parameter.vSubsample))/3;
        sizeBefore = (int)(192*144*3*8*uvFaktor);
        sizeAfter = yBlockFolge.lengthVLICoded +
                    crBlockFolge.lengthVLICoded +
                    cbBlockFolge.lengthVLICoded;
        
	    compressionRatio = (int)(100.0 * (float)sizeAfter / (float)sizeBefore);
        
        vec = new Vector();
        vec.addElement(new OurImage(paintVLIBar(450, 150, Color.blue)));
        vec.addElement(new Integer(compressionRatio));
        return vec;
	}

    /**
     * Zeichnet die Symbollngen aller Huffman Symbole des markierten Y-Blocks
     * als Balkendiagramm
     * @author Hari
     * @version 1
     * @param int width
     * @param int height
     * @param Color
     * @returns BufferedImage
     */
    public BufferedImage paintHuffmanSizes(int width, int height, Color colorDC, Color colorAC, Color colorAvgBefore, Color colorAvgAfter) {
        int blockIdx = deMCUOrder[(markerY/8)*xBlockx(1) + markerX/8];
        SymbolBlock block = yBlockFolge.elementAt(blockIdx);
        return block.paintHuffmanSizes(width, height, colorDC, colorAC, colorAvgBefore, colorAvgAfter);
    }
    
    /**
     * Zeichnet die Lngen der Huffman Symbole als Histogramm.
     * @author Hari
     * @version 1
     * @param int width
     * @param int height
     * @param Color
     * @returns BufferedImage
     */
    public BufferedImage paintAfterHuffman(int width, int height, Color c) {
        Histogramm histo;
        histo = yBlockFolge.getHuffmanHistogramm();
        histo.addHisto(crBlockFolge.getHuffmanHistogramm());
        histo.addHisto(cbBlockFolge.getHuffmanHistogramm());
        return histo.bitlaengenHistogramm(width, height, c);
    }
    
    /**
     * Zeichnet die Lngen der Huffman Tabellen als Histogramm.
     * @author Hari
     * @version 1
     * @param int width
     * @param int height
     * @param Color
     * @returns BufferedImage
     */
    public BufferedImage paintHuffmanTables(int width, int height, Color c) {
        Histogramm histo;
        
        histo = dcLumHuffmanTable.getHistogramm();
        histo.addHisto(dcChromHuffmanTable.getHistogramm());
        histo.addHisto(acLumHuffmanTable.getHistogramm());
        histo.addHisto(acChromHuffmanTable.getHistogramm());
        return histo.bitlaengenHistogramm(width, height, c);
    }
    
    /**
     * Codiert das Bild neu und liefert einen <CODE>Vector</CODE> zurck, der
     * die 2 Huffman Statistiken und den Huffman Kompressionsfaktor enthlt.
     * Wird von <CODE>PanelEntropie</CODE> benutzt.
     * @author  Hari
     * @version 1 - 15.1.2000
     * @returns Vector
     * @see     PanelEntropie
     */
    public Vector getHuffmanStats() {
        int sizeBefore, sizeAfter;
        int huffmanRatio;
        Vector vec;
        
        // ganzes Bild neu berechnen und codieren, um die 3 SymbBlockFolgen zu erhalten
        quantBloeckeErzeugen(FrameVisJPEG.vis.parameter);
        symboleErzeugen(FrameVisJPEG.vis.parameter);
        symboleCodieren(FrameVisJPEG.vis.parameter);
        
        sizeBefore = yBlockFolge.lengthVLICoded +
                     crBlockFolge.lengthVLICoded +
                     cbBlockFolge.lengthVLICoded;
        sizeAfter = lengthHuffmanCoded;

        huffmanRatio = 100 * sizeAfter / sizeBefore;
        
        vec = new Vector();
        vec.addElement(new OurImage(paintHuffmanTables(450, 90, Color.blue)));
        vec.addElement(new OurImage(paintAfterHuffman(450, 90, Color.blue)));
        vec.addElement(new Integer(huffmanRatio));
        return vec;
    }
}