/**
 * Die Klasse <CODE>JPEGParameter</CODE> kapselt alle Eingabeparameter des JPEG-Algorithmus.
 * In Zukunft kann die Klasse noch um Methoden zum Speichern und Wiedereinlesen erweitert
 * werden, damit komplette Parameterstze zwischengespeichert werden knnen.
 * @author  Raimund Specht
 * @version 3-07.12.1999
 */
public class JPEGParameter {
    /** 
     * Die Standard-Quantisierungsmatrix fr Helligkeitskomponenten (laut JPEG-Standard). Die 
     * Matrix sieht wie folgt aus:<BR>
     * <PRE>16   11   10   16   24   40   51   61
            12   12   14   19   26   58   60   55
            14   13   16   24   40   57   69   56
            14   17   22   29   51   87   80   62
            18   22   37   56   68  109  103   77
            24   35   55   64   81  104  113   92
            49   64   78   87  103  121  120  101
            72   92   95   98  112  100  103   99</PRE>
     */
    public final static Matrix STANDARD_QMATRIX_LUM;
    /** 
     * Die Standard-Quantisierungsmatrix fr Farbigkeitskomponenten (laut JPEG-Standard). Die
     * Matrix sieht wie folgt aus:<BR>
     * <PRE>17   18   24   47   99   99   99   99
            18   21   26   66   99   99   99   99
            24   26   56   99   99   99   99   99
            47   66   99   99   99   99   99   99
            99   99   99   99   99   99   99   99
            99   99   99   99   99   99   99   99
            99   99   99   99   99   99   99   99
            99   99   99   99   99   99   99   99</PRE>
     */
    public final static Matrix STANDARD_QMATRIX_CHROM;
    /** 
     * Eine Quantisierungsmatrix mit identischen Faktoren fr alle Elemente
     */
    public final static Matrix UNIFORM_QMATRIX;
    /** 
     * Eine Quantisierungsmatrix mit von links oben nach rechts unten wachsenden Faktoren
     */
    public final static Matrix NONUNIFORM_QMATRIX;
    /** 
     * Konstante zur Auswahl des Entropie-Kodierungsalgorithmus: Huffman-Verfahren mit vorberechnetem Baum
     */
    public final static int HUFFMAN_STANDARD = 1;
    /** 
     * Konstante zur Auswahl des Entropie-Kodierungsalgorithmus: Huffman-Verfahren mit individuell berechnetem Baum
     */
    public final static int HUFFMAN_ADAPTIVE = 2;
    /** 
     * Konstante zur Auswahl des Entropie-Kodierungsalgorithmus: Arithmetische Kodierung
     */
    public final static int ARITHMETIC = 3;
    
    /** 
     * <CODE>static</CODE>-Initialisierung der vier Quantisierungsmatrizen
     */
    static {
        // Standard-Quantisierungsmatrix fr Helligkeit aufbauen
        // (siehe Kap. 4.2 von "Multimedia Systems" (Link auf Praktikumshomepage))
        // geht das einfacher ??
        // Anm. von Hari: Jaa Raimund, das geht einfacher!!! ;-)
        
        STANDARD_QMATRIX_LUM = new Matrix();

        STANDARD_QMATRIX_LUM.matrix[0][0] = 16;
        STANDARD_QMATRIX_LUM.matrix[1][0] = 11;
        STANDARD_QMATRIX_LUM.matrix[2][0] = 10;
        STANDARD_QMATRIX_LUM.matrix[3][0] = 16;
        STANDARD_QMATRIX_LUM.matrix[4][0] = 24;
        STANDARD_QMATRIX_LUM.matrix[5][0] = 40;
        STANDARD_QMATRIX_LUM.matrix[6][0] = 51;
        STANDARD_QMATRIX_LUM.matrix[7][0] = 61;

        STANDARD_QMATRIX_LUM.matrix[0][1] = 12;
        STANDARD_QMATRIX_LUM.matrix[1][1] = 12;
        STANDARD_QMATRIX_LUM.matrix[2][1] = 14;
        STANDARD_QMATRIX_LUM.matrix[3][1] = 19;
        STANDARD_QMATRIX_LUM.matrix[4][1] = 26;
        STANDARD_QMATRIX_LUM.matrix[5][1] = 58;
        STANDARD_QMATRIX_LUM.matrix[6][1] = 60;
        STANDARD_QMATRIX_LUM.matrix[7][1] = 55;

        STANDARD_QMATRIX_LUM.matrix[0][2] = 14;
        STANDARD_QMATRIX_LUM.matrix[1][2] = 13;
        STANDARD_QMATRIX_LUM.matrix[2][2] = 16;
        STANDARD_QMATRIX_LUM.matrix[3][2] = 24;
        STANDARD_QMATRIX_LUM.matrix[4][2] = 40;
        STANDARD_QMATRIX_LUM.matrix[5][2] = 57;
        STANDARD_QMATRIX_LUM.matrix[6][2] = 69;
        STANDARD_QMATRIX_LUM.matrix[7][2] = 56;

        STANDARD_QMATRIX_LUM.matrix[0][3] = 14;
        STANDARD_QMATRIX_LUM.matrix[1][3] = 17;
        STANDARD_QMATRIX_LUM.matrix[2][3] = 22;
        STANDARD_QMATRIX_LUM.matrix[3][3] = 29;
        STANDARD_QMATRIX_LUM.matrix[4][3] = 51;
        STANDARD_QMATRIX_LUM.matrix[5][3] = 87;
        STANDARD_QMATRIX_LUM.matrix[6][3] = 80;
        STANDARD_QMATRIX_LUM.matrix[7][3] = 62;

        STANDARD_QMATRIX_LUM.matrix[0][4] = 18;
        STANDARD_QMATRIX_LUM.matrix[1][4] = 22;
        STANDARD_QMATRIX_LUM.matrix[2][4] = 37;
        STANDARD_QMATRIX_LUM.matrix[3][4] = 56;
        STANDARD_QMATRIX_LUM.matrix[4][4] = 68;
        STANDARD_QMATRIX_LUM.matrix[5][4] = 109;
        STANDARD_QMATRIX_LUM.matrix[6][4] = 103;
        STANDARD_QMATRIX_LUM.matrix[7][4] = 77;

        STANDARD_QMATRIX_LUM.matrix[0][5] = 24;
        STANDARD_QMATRIX_LUM.matrix[1][5] = 35;
        STANDARD_QMATRIX_LUM.matrix[2][5] = 55;
        STANDARD_QMATRIX_LUM.matrix[3][5] = 64;
        STANDARD_QMATRIX_LUM.matrix[4][5] = 81;
        STANDARD_QMATRIX_LUM.matrix[5][5] = 104;
        STANDARD_QMATRIX_LUM.matrix[6][5] = 113;
        STANDARD_QMATRIX_LUM.matrix[7][5] = 92;

        STANDARD_QMATRIX_LUM.matrix[0][6] = 49;
        STANDARD_QMATRIX_LUM.matrix[1][6] = 64;
        STANDARD_QMATRIX_LUM.matrix[2][6] = 78;
        STANDARD_QMATRIX_LUM.matrix[3][6] = 87;
        STANDARD_QMATRIX_LUM.matrix[4][6] = 103;
        STANDARD_QMATRIX_LUM.matrix[5][6] = 121;
        STANDARD_QMATRIX_LUM.matrix[6][6] = 120;
        STANDARD_QMATRIX_LUM.matrix[7][6] = 101;

        STANDARD_QMATRIX_LUM.matrix[0][7] = 72;
        STANDARD_QMATRIX_LUM.matrix[1][7] = 92;
        STANDARD_QMATRIX_LUM.matrix[2][7] = 95;
        STANDARD_QMATRIX_LUM.matrix[3][7] = 98;
        STANDARD_QMATRIX_LUM.matrix[4][7] = 112;
        STANDARD_QMATRIX_LUM.matrix[5][7] = 100;
        STANDARD_QMATRIX_LUM.matrix[6][7] = 103;
        STANDARD_QMATRIX_LUM.matrix[7][7] = 99;
        
        // Farbigkeitsquantisierungsmatrix aufbauen
        STANDARD_QMATRIX_CHROM = new Matrix();
        for (int i=0; i<8; i++)
           for (int j=0; j<8; j++) STANDARD_QMATRIX_CHROM.matrix[i][j] = 99;
        STANDARD_QMATRIX_CHROM.matrix[0][0] = 17;
        STANDARD_QMATRIX_CHROM.matrix[1][0] = 18;
        STANDARD_QMATRIX_CHROM.matrix[2][0] = 24;
        STANDARD_QMATRIX_CHROM.matrix[3][0] = 47;

        STANDARD_QMATRIX_CHROM.matrix[0][1] = 18;
        STANDARD_QMATRIX_CHROM.matrix[1][1] = 21;
        STANDARD_QMATRIX_CHROM.matrix[2][1] = 26;
        STANDARD_QMATRIX_CHROM.matrix[3][1] = 66;

        STANDARD_QMATRIX_CHROM.matrix[0][2] = 24;
        STANDARD_QMATRIX_CHROM.matrix[1][2] = 26;
        STANDARD_QMATRIX_CHROM.matrix[2][2] = 56;

        STANDARD_QMATRIX_CHROM.matrix[0][3] = 47;
        STANDARD_QMATRIX_CHROM.matrix[1][3] = 66;

        // konstante Quantisierungsmatrix aufbauen
        UNIFORM_QMATRIX = new Matrix();
        for (int i=0; i<8; i++)
           for (int j=0; j<8; j++) UNIFORM_QMATRIX.matrix[i][j] = 50;

        // nicht-konstante Quantisierungsmatrix aufbauen
        NONUNIFORM_QMATRIX = new Matrix();
        for (int i=0; i<8; i++)
           for (int j=0; j<8; j++) NONUNIFORM_QMATRIX.matrix[i][j] = 20 + 5*i + 5*j;
    }  // static

    /**
     * Gibt das Farbsubsampling fr die Chrominanzkomponenten in horizontaler Richtung an (1 = kein Subsampling, 2 = halbe Gre, usw.)
     */
    public int hSubsample;
    /**
     * Gibt das Farbsubsampling fr die Chrominanzkomponenten in horizontaler Richtung an (1 = kein Subsampling, 2 = halbe Gre, usw.)
     */
    public int vSubsample;
    /**
     * Speichert die Quantisierungsmatrix, die fr die Helligkeitskomponente verwendet wird.
     */
    public Matrix qMatrixLum;
    /**
     * Speichert die Quantisierungsmatrix, die fr die Farbigkeitskomponenten verwendet wird.
     */
    public Matrix qMatrixChrom;
    /**
     * Speichert den aktuellen Quantisierungsfaktor. Dieser wird zur Skalierung der Quantisierungsmatrix
     * fr die Farbkomponenten bentigt. In <CODE>qMatrixChrom</CODE> (Farbkomponente) ist der Faktor
     * bereits eingerechnet!
     */
    public int qFaktor;
    /**
     * Speichert die Art der verwendeten Entropie-Kodierung.
     * @see #HUFFMAN_STANDARD
     * @see #HUFFMAN_ADAPTIVE
     * @see #ARITHMETIC
     */
    public int compressionType;
    
    /**
     * Name der Datei des Originalbildes
     */
    public String filename;
    
    /**
     * Zu verwendender Skalierungsalgorithmus fr die Farbkanle
     * (0=schnell/schlecht, 1=mittel, 2=langsam/gut)
     */
    public int skalierungsQualitaet;
    
    /**
     * x-Offset des zu komprimierenden Bildes im geladenen Rohbild
     */
	int xShift;
	
    /**
     * y-Offset des zu komprimierenden Bildes im geladenen Rohbild
     */
	int yShift;
	
    /**
     * Erzeugt ein neues Objekt, das mit Standardeinstellungen geladen wird: Standard-Quantisierungsmatrix,
     * Huffman-Kompression mit statischem Baum und 2x2 Farbsubsampling.
     * @author  Raimund Specht
     * @version 1-05.12.1999
     */
    public JPEGParameter() {
        qFaktor = 50;
        qMatrixLum = (Matrix)STANDARD_QMATRIX_LUM.clone();
        qMatrixLum.scale(qFaktor);
        qMatrixChrom = (Matrix)STANDARD_QMATRIX_CHROM.clone();
        qMatrixChrom.scale(qFaktor);
        compressionType = HUFFMAN_STANDARD;
        hSubsample = 2;
        vSubsample = 2;
        filename = FrameVisJPEG.standardBilder[0];
        skalierungsQualitaet = 1;
        xShift = 0;
        yShift = 0;
    } 
    
    /**
     * Klont das <CODE>Parameter</CODE> Objekt mit
     * Hilfe der Gentechnik. Hahahaaa war nur ein Witz.
     * @author  Hari
     * @version 1 - 30.12.1999
     * @returns Object
     */
    public Object clone() {
    	JPEGParameter parameter;
    	
    	parameter = new JPEGParameter();
    	parameter.hSubsample = hSubsample;
    	parameter.vSubsample = vSubsample;
    	parameter.qFaktor = qFaktor;
    	parameter.compressionType = compressionType;
    	parameter.qMatrixLum = (Matrix)qMatrixLum.clone();
    	parameter.qMatrixChrom = (Matrix)qMatrixChrom.clone();
    	parameter.filename = filename;
    	parameter.skalierungsQualitaet = skalierungsQualitaet;
    	parameter.xShift = xShift;
    	parameter.yShift = yShift;
    	return parameter;
    }
       
    /**
     * Vergleicht das Objekt mit einem anderen <CODE>Parameter</CODE> Objekt
     * und gibt <CODE>true</CODE> zurck, falls die beiden gleich sind
     * @author  Hari
     * @author  Raimund Specht (mu "parameter" vom Typ Object haben und mit parameter == null zurecht kommen)
     * @version 2 - 03.01.2000
     * @parameter JPEGParameter
     * @returns boolean
     */
    public boolean equals(Object parameter) {
        int i;
        JPEGParameter testParameter;

        if (parameter == null) return false;
        if (!(parameter instanceof JPEGParameter)) return false;
        testParameter = (JPEGParameter)parameter;
        
        if (hSubsample != testParameter.hSubsample)
                return false;
        if (vSubsample != testParameter.vSubsample)
                return false;
        if (qFaktor != testParameter.qFaktor)
                return false;
        if (compressionType != testParameter.compressionType)
                return false;
        if (!qMatrixLum.equals(testParameter.qMatrixLum))
                return false;
        if (!qMatrixChrom.equals(testParameter.qMatrixChrom))
                return false;
        if (filename.compareTo(testParameter.filename) != 0)
                return false;
        if (skalierungsQualitaet != testParameter.skalierungsQualitaet)
        	    return false;
		if (xShift != testParameter.xShift)
				return false;
		if (yShift != testParameter.yShift)
				return false;
        return true;
    }
}