import javax.help.*;
import java.net.URL;

/**
 * <CODE>JavaHelpSystem</CODE> bildet die Schnittstelle zu JavaHelp, das
 * fr die Anzeige der online-Hilfe zustndig ist. <CODE>JavaHelpSystem</CODE>
 * kann nicht instantiiert werden; es stellt im wesentlichen nur eine
 * Referenz auf ein <CODE>HelpBroker</CODE>-Objekt zur Verfgung.
 * Damit die Online-Hilfe funktioniert, mu das Archiv <CODE>jh.jar</CODE>
 * im <CODE>CLASSPATH</CODE> angegeben sein.
 * @author  Raimund Specht
 * @version 1-17.01.2000
 */
public class JavaHelpSystem {
    /**
     * <CODE>helpSystemAvailable</CODE> zeigt an, ob JavaHelp erfolgreich
     * initialisiert werden konnte, sprich ob das Online-Hilfe-System zur
     * Verfgung steht. Falls etwa das HelpSet nicht gefunden werden konnte,
     * kann keine Online-Hilfe verwendet werden.
     */
    public static boolean helpSystemAvailable = false;
    /** 
     * Der <CODE>HelpBroker</CODE> bildet die Schnittstelle zu JavaHelp. Diese
     * Referenz wird fr Aufrufe von <CODE>CSH.DisplayHelpFromSource()</CODE> 
     * bentigt.
     */
    public static HelpBroker hb;
    private static HelpSet hs;
    
    /**
     * Statische Initialisierungsroutine
     */
    static {
        ClassLoader loader = FrameVisJPEG.vis.getClass().getClassLoader();
        try {
            URL hsURL = HelpSet.findHelpSet(loader, "help/jpeg.hs");
            hs = new HelpSet(loader, hsURL);
            hb = hs.createHelpBroker();
            helpSystemAvailable = true;
        } catch (Exception ee) { 
            System.out.println("HelpSet nicht gefunden.");
            helpSystemAvailable = false;
        }
    } // static

} // Klasse JavaHelpSystem