import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.util.*;


/**
 * Die Klasse <CODE>Matrix</CODE> kapselt im Prinzip ein <CODE>int[8][8]</CODE>-Array. Der Haupteinsatzzweck
 * der Klasse ist es, einen Typ fr Methodenparameter bereitzustellen:<BR>
 * <CODE>Matrix myMethod(Matrix m)</CODE> sieht besser aus und ist flexibler als
 * <CODE>int[][] myMethod(int[][] m)</CODE>.<BR>
 * Daneben verfgt <CODE></CODE> noch ber Methoden zur Skalierung und zur Darstellung in einer 
 * <CODE>JTable</CODE>. Da die Klasse ein <CODE>int</CODE>-Array kapselt und keine wesentlichen
 * Methoden zu ihrer Manipulation bereitstellt, ist sie vor allem zur Speicherung
 * von Quantisierungsmatrizen gedacht. Deshalb wird der Wertebereich der Matrix-Elemente auf <CODE>1..255</CODE>
 * beschrnkt (Baseline Codec).
 * Die Klasse implementiert das Interface <CODE>TableModel</CODE>, um auf die Darstellung in einer
 * <CODE>JTable</CODE> beeinflussen und auf Benutzerinteraktionen reagieren zu knnen.
 * @author  Raimund Specht
 * @see     javax.swing.table.TableModel
 */
public class Matrix implements TableModel {

    /**
     * Dieses Feld speichert die zugrundeliegende 8x8-Matrix als <CODE>int[][]</CODE>-Array. Der erste Index stellt die
     * x-Richtung dar, der zweite die y-Richtung.
     */
    public int[][] matrix;
    private Vector myTableModelListeners;

    /**
     * Legt ein neues "leeres" <CODE>Matrix</CODE>-Objekt an. Jedes Element wird mit dem Wert 1 initialisiert, damit
     * ein im Baseline Codec gltiger Wert erreicht wird.
     * @author  Raimund Specht
     * @version 2-07.12.1999
     */
    public Matrix() {
        int i, j;
        
        matrix = new int[8][8];
        for (i = 0; i<8; i++)
            for (j = 0; j<8; j++)
                matrix[i][j] = 1;
        
        myTableModelListeners = new Vector();
    }

    /**
     * Legt ein neues <CODE>Matrix</CODE>-Objekt an, wobei die Matrix mit den Werten des als Parameter bergebenen
     * <CODE>int[][]</CODE>-Arrays gefllt wird. Die Werte werden ggf. auf den Wertebereich <CODE>1..255</CODE> beschrnkt.
     * @author  Raimund Specht
     * @version 1-02.12.1999
     * @param   matrix
     */
    public Matrix(int[][] matrix) {
        int i, j;
        
        this.matrix = new int[8][8];
        for (i=0; i<8; i++)
            for (j=0; j<8; j++) {
                this.matrix[i][j] = matrix[i][j];
                if (this.matrix[i][j] <= 0) this.matrix[i][j] = 1;
                if (this.matrix[i][j] > 255) this.matrix[i][j] = 255;
            }
        myTableModelListeners = new Vector();
    }

    /**
     * Liefert eine identische Kopie (Klon, AKA Dolly) des Objekts zurck.
     * @author  Raimund Specht
     * @version 2-06.12.1999
     * @returns Object
     */
    public Object clone() {
        Matrix newMatrix;
        
        newMatrix = new Matrix(matrix);
        newMatrix.myTableModelListeners = (Vector)myTableModelListeners.clone();
        return newMatrix;
    }

    /**
     * Gibt eine neue <CODE>Matrix</CODE> zurck, in der die Werte in
     * Zickzack Reihenfolge stehen.
     * Jaja ich wei, die Methode gibts schon irgendwo, aber dafr ist
     * die Routine hier auch ineffizienter, da ohne Tabelle.
     * @author  Hari
     * @version 1 - 31.1.2000
     * @returns Matrix
     */
    public Matrix getZigZagOrdered() {
        int matrixNeu[][] = new int[8][8];
        int x, y, i, j;
        int richtung = 1;  // 0=nach linx untn   1=nach rechz obm

        x = 0;
        y = 0;
        i = 0;
        j = 0;
        while (x<8 && y<8) {
            matrixNeu[i][j] = matrix[x][y];
            
            if (richtung == 1)
                if (y==0) {
                    richtung = 0;
                    if (x==7)
                        y++;
                    else
                        x++;
                }
                else if (x==7) {
                    richtung = 0;
                    y++;
                }
                else {
                    x++;
                    y--;
                }
            else
                if (x==0) {
                    richtung = 1;
                    if (y==7)
                        x++;
                    else
                        y++;
                }
                else if (y==7) {
                    x++;
                    richtung = 1;
                }
                else {
                    x--;
                    y++;
                }
                
            if (i == 7) {
                i = 0;
                j++;
            }
            else
                i++;
        }
        
        return new Matrix(matrixNeu);
    }
    
    /**
     * Diese Methode skaliert die gesamte Matrix, d.h. jedes Element, mit einem Faktor (Multiplikation). Falls die
     * Werte durch die Skalierung den im Baseline Codec erlaubten Wertebereich (1..255) verlassen wrden, werden sie 
     * automatisch angepat.
     * @author  Hari
     * @version 1-03.12.1999
     * @param   int qualFaktor Der gewnschte Qualittsfaktor; Wertebereich von 1..100
     */
    public void scale(int qualFaktor) {
        int i, j;
        int qualFaktor2;
        
        if (qualFaktor < 50)
            qualFaktor2 = 5000 / qualFaktor;
        else
            qualFaktor2 = 200 - qualFaktor*2;
        
        for (i=0; i<8; i++)
            for (j=0; j<8; j++) {
                int temp = (matrix[i][j]*qualFaktor2 + 50) / 100;
                if (temp <= 0)
                    temp = 1;
                if (temp > 255)
                    temp = 255;
                matrix[i][j] = temp;
            }
    }

    // alte Version, macht einfache Multiplikation
/*    public void scale(float faktor) {
        int i, j;
        
        for (i=0; i<8; i++)
            for (j=0; j<8; j++) {
                matrix[i][j] = (int)(matrix[i][j] * faktor);
                if (matrix[i][j] <= 0) matrix[i][j] = 1;
                if (matrix[i][j] > 255) matrix[i][j] = 255;  // Baseline Codec: QMatrix-Werte:  1..255
            }
    }*/


    /**
     * Diese Methode fgt den bergebenen <CODE>TableModelListener</CODE> zu der Liste der Listener hinzu. 
     * Er wird fortan benachrichtigt, wenn der Benutzer nderungen an der in einer <CODE>JTable</CODE> 
     * dargestellten Matrix vornimmt. Implementiert <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-05.12.1999
     * @param   l der hinzuzufgende <CODE>TableModelListener</CODE>
     * @see     javax.swing.table.TableModel
     */
    public void addTableModelListener(TableModelListener l) {
        myTableModelListeners.add(l);
    }
    
    /**
     * Diese Methode entfernt den bergebenen <CODE>TableModelListener</CODE> aus der Liste der
     * Listener. Implementiert <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-05.12.1999
     * @param   l der zu entfernende <CODE>TableModelListener</CODE>
     * @see     javax.swing.table.TableModel
     */
    public void removeTableModelListener(TableModelListener l) {
        myTableModelListeners.removeElement(l);
    }
    
    /**
     * Diese Methode ndert einen Eintrag des Feldes <CODE>matrix</CODE>, wenn der Benutzer die
     * in einer <CODE>JTable</CODE> dargestellte Matrix verndert. Falls kein gltiger Integer
     * eingegeben wurde, bleibt der ursprngliche Wert der Zelle erhalten. Falls Null oder ein
     * negativer Wert eingegeben wurde, wird der Wert stattdessen auf <CODE>1</CODE> gesetzt.
     * Implementiert <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-05.12.1999
     * @param   aValue      Der neue Wert der Zelle, als <CODE>java.lang.Object</CODE>
     * @param   rowIndex    Zeile, in der sich die zu ndernde Zelle befindet
     * @param   columnIndex Spalte, in der sich die zu ndernde Zelle befindet
     * @see     javax.swing.table.TableModel
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        int i;
        
        try {
            matrix[columnIndex][rowIndex] = Integer.parseInt((String)aValue);
            if (matrix[columnIndex][rowIndex] <= 0) matrix[columnIndex][rowIndex] = 1;
            if (matrix[columnIndex][rowIndex] > 255) matrix[columnIndex][rowIndex] = 255;  // Baseline Codec-Wertebereich

            // Listener ber die nderung informieren            
            for (i=0; i<myTableModelListeners.size(); i++) 
                ((TableModelListener)myTableModelListeners.elementAt(i)).tableChanged(new TableModelEvent(this));
        } catch (NumberFormatException e) { }
    }
    
    /**
     * Implementiert <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-03.12.1999
     * @param   rowIndex Zeile, in der sich die auszulesende Zelle befindet
     * @param   columnIndex Spalte, in der sich die auszulesende Zelle befindet
     * @returns java.lang.Object
     * @see     javax.swing.table.TableModel
     */
    public Object getValueAt(int rowIndex, int columnIndex) { 
        return new Integer(matrix[columnIndex][rowIndex]);
    }
    
    /**
     * Mit diese Methode kann fr eine beliebige Zelle berprft werden, ob diese Zelle vom Benutzer
     * verndert werden kann, wenn die Matrix in einer <CODE>JTable</CODE> dargestellt wird. Die Klasse
     * <CODE>Matrix</CODE> liefert hier immer <CODE><B>true</B></CODE> zurck, d.h. jede Zelle kann
     * bearbeitet werden. Implementiert <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-03.12.1999
     * @param   rowIndex Zeile der angesprochenen Zelle
     * @param   columnIndex Spalte der angesprochenen Zelle
     * @returns boolean
     * @see     javax.swing.table.TableModel
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return true;
    }

    /**
     * Implementiert das Interface <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-03.12.1999
     * @param   columnIndex
     * @returns Class
     * @see     javax.swing.table.TableModel
     */
    public Class getColumnClass(int columnIndex) {
        Integer i = new Integer(3);
        return i.getClass();
    }
    
    /**
     * Implementiert das Interface <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-03.12.1999
     * @param   columnIndex
     * @returns String
     * @see     javax.swing.table.TableModel
     */
    public String getColumnName(int columnIndex) {
        return "";
    }
    
    /**
     * Liefert die Anzahl der Spalten der Matrix zurck (8). Implementiert <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-03.12.1999
     * @returns int
     * @see     javax.swing.table.TableModel
     */
    public int getColumnCount() {
        return 8;
    }
    
    /**
     * Liefert die Anzahl der Zeilen der Matrix zurck (8). Implementiert <CODE>TableModel</CODE>.
     * @author  Raimund Specht
     * @version 1-03.12.1999
     * @returns int
     * @see     javax.swing.table.TableModel
     */
    public int getRowCount() {
        return 8;
    }
    
    /**
     * Gibt eine Darstellung der Matrix als <CODE>JTable</CODE> zurck.
     * @author  Raimund Specht
     * @version 1-02.12.1999
     * @param   farbe Hintergrundfarbe, mit der die <CODE>JTable</CODE> dargestellt werden soll
     * @returns javax.swing.JTable
     */
    public JTable paintMatrix(Color farbe) {
        JTable table;
        
        table = new JTable(this);
        table.setBackground(farbe);
        table.setFont(GUIParameter.SMALL_FONT);
        table.setPreferredSize(new Dimension(192, table.getPreferredSize().height));
        table.setMaximumSize(new Dimension(192, table.getPreferredSize().height));
        table.setMinimumSize(new Dimension(192, table.getPreferredSize().height));
        return table;
    }
    
    /**
     * Vergleicht die Matrix mit einer zweiten. Falls gleich, wird
     * <CODE>true</CODE> zurckgeliefert
     * @author  Hari
     * @version 1 - 30.12.1999
     * @param   Matrix
     * @returns boolean
     */
    public boolean equals(Matrix matrix) {
    	int i, j;
    	
    	for (i=0; i<8; i++)
    		for (j=0; j<8; j++)
    			if (this.matrix[i][j] != matrix.matrix[i][j])
    				return false;
    	return true;
    }
 }