import java.awt.*;
import java.awt.image.*;
import com.sun.image.codec.jpeg.*;

/**
 * Die Klasse <CODE>OurHuffmanTable</CODE> kapselt eine Tabelle fr Lngen der Huffman Symbole.
 * Die Tabelle ist fr AC mit dem Wert run*16+size indiziert, bzw. mit size fr DC.
 * Die Klasse unterscheidet sich von <CODE>com.sun.image.codec.jpeg.JPEGHuffmanTable</CODE>
 * dadurch, dass <CODE>OurHuffmanTable</CODE> direkt fr ein gegebenes Huffman Symbol die
 * Lnge ausspucken kann, whrend bei <CODE>JPEGHuffmanTable</CODE> erst rumdecodiert werden muss.
 * Zugegriffen wird auf die Tabelle ber die Arrays <CODE>lengths</CODE>
 * und <CODE>symbols</CODE>
 * Es gibt einen Konstruktor, der eine <CODE>OurHuffmanTable</CODE> kann aus
 * einer <CODE>JPEGHuffmanTable</CODE> erzeugt.
 * Auerdem einen leeren Konstruktor, damit man eine eigene Huffman Tabelle nach
 * Hufigkeiten erzeugen kann.
 * Die Konstruktoren setzen nicht bentigte (bzw. alle, beim leeren Konstruktor)
 * Eintrge in der Lngentabelle auf -1, damit die bei der Darstellung der Tabelle
 * ausgesondert werden knnen.
 * @author  Hari
 */
class OurHuffmanTable {
    public int symbols[];
    public int lengths[];
    public short codedSymbols[];
    public short codedLengths[];

    /**
     * Erzeugt eine leere <CODE>OurHuffmanTable</CODE>
     * @author  Hari
     * @version 1 - 30.12.99
     */
	public OurHuffmanTable() {
		lengths = new int[256];
		for (int i=0; i<256; i++)
			lengths[i] = -1;
        symbols = new int[256];    
	}
    
    /**
     * Erzeugt eine <CODE>OurHuffmanTable</CODE> aus einer <CODE>JPEGHuffmanTable</CODE>
     * @author  Hari
     * @version 1 - 27.12.99
     * @param JPEGHuffmanTable
     */
	public OurHuffmanTable(JPEGHuffmanTable huffTable) {
		codedLengths = huffTable.getLengths();	
		codedSymbols = huffTable.getSymbols();
		int numSymbols = codedSymbols.length;
		int decodedLengths[] = new int[numSymbols];
		int i, j, pos;
		
		pos = 0;
		for (i=0; i<codedLengths.length; i++)
			for (j=0; j<codedLengths[i]; j++) {
				decodedLengths[pos] = i;
				pos++;
			}

		lengths = new int[256];
        for (i=0; i<256; i++)
			lengths[i] = -1;
        for (i=0; i<numSymbols; i++)
            lengths[codedSymbols[i]] = decodedLengths[i];
        
        // Huffman Symbole erzeugen
        symbols = new int[256];
        int code = 0;
        i = 0;
        int size = 0;
        while (i<numSymbols) {
            while (i<numSymbols && decodedLengths[i]==size) {
                symbols[codedSymbols[i]] = code;
                code++;
                i++;
            }    
            code <<= 1;
            size++;
        }
	}
	
	/**
	 * Vergleicht die Tabelle mit einer zweiten. Falls ja, wird
	 * <CODE>true</CODE> zurckgeliefert. Wir vergleichen mal nur
     * die Lngentabelle, das reicht mit "an Sicherheit grenzender
     * Wahrscheinlichkeit" aus.
     * @author  Hari
     * @version 1 - 30.12.99
     * @param OurHuffmanTable
     * @returns boolean
     */
	public boolean equals(OurHuffmanTable huffTable) {
		int i;
		
		for (i=0; i<lengths.length; i++)
			if (lengths[i] != huffTable.lengths[i])
				return false;
		return true;
	}

    /**
     * Zeichnet fr jede Symbollnge die Anzahl Huffman Symbole in der Tabelle
     * @author Hari
     * @version 1 - 5.1.00
     * @param int width
     * @param int height
     * @param Color
     * @returns BufferedImage
     */
    public Histogramm getHistogramm() {
        Histogramm histo;
        int i;
        
        histo = new Histogramm(17);
        for (i=0; i<256; i++)
            if (lengths[i] != -1)
                histo.werte[lengths[i]]++;
        return histo;
    }

}