import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.help.*;

/**
 * Das Farbraum Panel. Es enthlt die drei RGB Kanle und die drei YUV Kanle.
 * Auerdem gibts eine Combobox fr den Skalierungsfaktor fr den Cr- und Cb Kanal.
 * @author  Raimund Specht, Hari
 */
public class PanelFarbraum extends JPanel
{
  private JPanel yImage, uImage, vImage, rotImage, gruenImage, blauImage, bar3DDCT, bar3DQuant, dynPanel;
  private JComboBox faktorCombo;
  private JComboBox algoCombo;
  private JProgressBar compressBar;
  private static String[] faktorComboStrings = {"ohne (1x1)", "2x1", "2x2", "4x2"};
  private static String[] algoComboStrings = {"niedrig (schnell)", "mittel", "hoch (langsam)"};
  private JPEGParameter parameter;
  
  /**
  * Konstruktor des Panels. Malt schomma' alle Komponenten drauf
  * @author  Hari
  */
  public PanelFarbraum()
  {
      super();
      
      setLayout(new BorderLayout());
      parameter = FrameVisJPEG.vis.parameter;
      addStaticComponents();
      addDynamicComponents();
      validate();
  }

  public void update()
  {
    dynPanel.removeAll();
    addDynamicComponents();
    validate();
  } // update()
	
	/**
	 * Liefert den lokalen Kompressionsfaktor zurck
	 * @author Hari
	 * @version 1 - 9.1.2000
	 * @returns int
	 */
	private int getCompressRatio() {
        int h = parameter.hSubsample;
        int v = parameter.vSubsample;
        return (100+200/(h*v))/3;
	}
	
  /**
  * Wird aufgerufen, wenn jemand etwas aus der Skalierungs-Combobox auswhlt
  * @author  Hari
  */
  private void comboBoxAuswahl()
  {
    String algoAuswahl;
	String faktorAuswahl;
	int hfaktor, vfaktor;
	
	faktorAuswahl = (String)faktorCombo.getSelectedItem();
	if (faktorAuswahl.equals(faktorComboStrings[0])) {
        hfaktor = 1;
        vfaktor = 1;
    }
	else if (faktorAuswahl.equals(faktorComboStrings[1])) {
        hfaktor = 2;
        vfaktor = 1;
    }
    else if (faktorAuswahl.equals(faktorComboStrings[2])) {
        hfaktor = 2;
        vfaktor = 2;
    }
	else {
        hfaktor = 4;
        vfaktor = 2;
    }
	if (hfaktor==1 && vfaktor==1)
		algoCombo.setEnabled(false);
	else
		algoCombo.setEnabled(true);
    parameter.hSubsample = hfaktor;
    parameter.vSubsample = vfaktor;
    int comRatio = getCompressRatio();
    compressBar.setValue(comRatio);
    compressBar.setString((new Integer(100-comRatio)).toString() + "%");
    
    algoAuswahl = (String)algoCombo.getSelectedItem();
    if (algoAuswahl.equals(algoComboStrings[0]))
    	parameter.skalierungsQualitaet = 0;
    if (algoAuswahl.equals(algoComboStrings[1]))
    	parameter.skalierungsQualitaet = 1;
    if (algoAuswahl.equals(algoComboStrings[2]))
    	parameter.skalierungsQualitaet = 2;

    update();
  } // comboBoxAuswahl()

  /**
  * Malt die Komponenten aufs Panel, die nur einmal am Anfang gemacht werden mssen,
  * und ruft <CODE>addDynamicComponents</CODE> auf, damit auch die Restlichen gezeichnet werden.
  * @author  Hari
  */
  private void addStaticComponents()
  {
    JLabel label;
    JButton backButton, vorButton, infoButton;
    JPanel p, p2, compressPanel, labelzPanel;

    p = new JPanel();
    p.setLayout(new BorderLayout());
    label = new JLabel(" Schritt 1: Farbraumkonvertierung");
    p.add(label, BorderLayout.WEST);
    p2 = new JPanel();

    backButton = new JButton("Zurck");
    backButton.setMnemonic(KeyEvent.VK_Z);
    backButton.setEnabled(false);
    p2.add(backButton);
      
    infoButton = new JButton("Info");
    infoButton.setMnemonic(KeyEvent.VK_I);
    if (JavaHelpSystem.helpSystemAvailable) {
        CSH.setHelpIDString(infoButton, "farbraum");
        infoButton.addActionListener(new CSH.DisplayHelpFromSource(JavaHelpSystem.hb));
    } else {
        infoButton.setEnabled(false);
        infoButton.setToolTipText("HelpSet nicht gefunden.");
    }
    p2.add(infoButton);

    vorButton = new JButton("Weiter");
    vorButton.setMnemonic(KeyEvent.VK_W);
    vorButton.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          FrameVisJPEG.vis.setActivePanel(1);
        }
      }
    );
    p2.add(vorButton);
    p.add(p2, BorderLayout.EAST);
    add(p, BorderLayout.NORTH);

    p2 = new JPanel();
    p2.setLayout(new BorderLayout());
    
    p = new JPanel();
    p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
    label = new JLabel("Cr/Cb Subsampling:");
    p.add(label);
    faktorCombo = new JComboBox(faktorComboStrings);
    switch ((new JPEGParameter()).hSubsample * (new JPEGParameter()).vSubsample) {
        case 1: faktorCombo.setSelectedIndex(0); break;
        case 2: faktorCombo.setSelectedIndex(1); break;
        case 4: faktorCombo.setSelectedIndex(2); break;
        default: faktorCombo.setSelectedIndex(3);
    }
    faktorCombo.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          comboBoxAuswahl();
        }
      }
    );
    p.add(faktorCombo);
    p2.add(p, BorderLayout.NORTH);
    
    p = new JPanel();
    p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
    label = new JLabel("Skalierungsqualitt:");
    p.add(label);
    algoCombo = new JComboBox(algoComboStrings);
    algoCombo.setSelectedIndex((new JPEGParameter()).skalierungsQualitaet);
    algoCombo.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          comboBoxAuswahl();
        }
      }
    );
    p.add(algoCombo);
    p2.add(p, BorderLayout.CENTER);

    compressPanel = new JPanel();
    compressPanel.setLayout(new BorderLayout());
    compressBar = new JProgressBar(JProgressBar.VERTICAL, 0, 100);
    int comRatio = getCompressRatio();
    compressBar.setValue(comRatio);
    compressBar.setString((new Integer(100-comRatio)).toString() + "%");
    compressBar.setStringPainted(true);
    
    labelzPanel = new JPanel();
    labelzPanel.setLayout(new BoxLayout(labelzPanel, BoxLayout.Y_AXIS));
    labelzPanel.add(new JLabel("Kompression in"));
    labelzPanel.add(new JLabel("diesem Schritt um:"));
    compressPanel.add(labelzPanel, BorderLayout.NORTH);
    
    p = new JPanel();
    p.add(compressBar);
    compressPanel.add(p, BorderLayout.CENTER);
    p2.add(compressPanel, BorderLayout.SOUTH);
    add(p2, BorderLayout.EAST);

    dynPanel = new JPanel();
    dynPanel.setLayout(new GridLayout(3, 2));
    // Komponenten werden in addDynamicComponents() hinzugefgt
    add(dynPanel, BorderLayout.CENTER);
  } // addStaticComponents()

  /**
   * Malt die Komponenten, die beim Update jedesmal neu gemacht werden mssen
   * @author  Hari
   */
  private void addDynamicComponents()
  {
    JPanel p2, p3;
    JLabel label;

    p2 = new JPanel();
    p3 = new JPanel();
    p2.setLayout(new BorderLayout());
    label = new JLabel(" Rotkanal:");
    p2.add(label, BorderLayout.NORTH);
    rotImage = new OurImage(FrameVisJPEG.vis.ourImage.getRImage());
    rotImage.setToolTipText("Rotanteil des Originalbildes");
    rotImage.setPreferredSize(new Dimension(192, 144));
    rotImage.setMinimumSize(new Dimension(192, 144));
    p3.setLayout(new GridBagLayout());
    p3.add(rotImage);
    p2.add(p3, BorderLayout.CENTER);
    dynPanel.add(p2);

    p2 = new JPanel();
    p3 = new JPanel();
    p2.setLayout(new BorderLayout());
    p2.add(new JLabel(" Y-Kanal:"), BorderLayout.NORTH);
    yImage = new OurImage(FrameVisJPEG.vis.ourImage.getYImage());
    yImage.setToolTipText("Helligkeitskomponente des Bildes");
    yImage.setPreferredSize(new Dimension(192, 144));
    yImage.setMinimumSize(new Dimension(192, 144));
    p3.setLayout(new GridBagLayout());
    p3.add(yImage);
    p2.add(p3, BorderLayout.CENTER);
    dynPanel.add(p2);

    p2 = new JPanel();
    p3 = new JPanel();
    p2.setLayout(new BorderLayout());
    p2.add(new JLabel(" Grnkanal:"), BorderLayout.NORTH);
    gruenImage = new OurImage(FrameVisJPEG.vis.ourImage.getGImage());
    gruenImage.setToolTipText("Grnanteil des Originalbildes");
    gruenImage.setPreferredSize(new Dimension(192, 144));
    gruenImage.setMinimumSize(new Dimension(192, 144));
    p3.setLayout(new GridBagLayout());
    p3.add(gruenImage);
    p2.add(p3, BorderLayout.CENTER);
    dynPanel.add(p2);

    p2 = new JPanel();
    p3 = new JPanel();
    p2.setLayout(new BorderLayout());
    p2.add(new JLabel(" Cr-Kanal:"), BorderLayout.NORTH);
    uImage = new OurImage(FrameVisJPEG.vis.ourImage.getIImage(parameter.hSubsample, parameter.vSubsample));
    uImage.setToolTipText(" Rot-Abweichung des Bildes");
    uImage.setPreferredSize(new Dimension(192/parameter.hSubsample, 144/parameter.vSubsample));
    uImage.setMinimumSize(new Dimension(192/parameter.hSubsample, 144/parameter.vSubsample));
    p3.setLayout(new GridBagLayout());
    p3.add(uImage);
    p2.add(p3);
    dynPanel.add(p2);

    p2 = new JPanel();
    p3 = new JPanel();
    p2.setLayout(new BorderLayout());
    p2.add(new JLabel(" Blaukanal:"), BorderLayout.NORTH);
    blauImage = new OurImage(FrameVisJPEG.vis.ourImage.getBImage());
    blauImage.setToolTipText("Blauanteil des Originalbildes");
    blauImage.setPreferredSize(new Dimension(192, 144));
    blauImage.setMinimumSize(new Dimension(192, 144));
    p3.setLayout(new GridBagLayout());
    p3.add(blauImage);
    p2.add(p3, BorderLayout.CENTER);
    dynPanel.add(p2);

    p2 = new JPanel();
    p3 = new JPanel();
    p2.setLayout(new BorderLayout());
    p2.add(new JLabel(" Cb-Kanal:"), BorderLayout.NORTH);
    vImage = new OurImage(FrameVisJPEG.vis.ourImage.getQImage(parameter.hSubsample, parameter.vSubsample));
    vImage.setToolTipText(" Blau-Abweichung des Bildes");
    vImage.setPreferredSize(new Dimension(192/parameter.hSubsample, 144/parameter.vSubsample));
    vImage.setMinimumSize(new Dimension(192/parameter.hSubsample, 144/parameter.vSubsample));
    p3.setLayout(new GridBagLayout());
    p3.add(vImage);
    p2.add(p3, BorderLayout.CENTER);
    dynPanel.add(p2);
  } // addDynamicComponents()
}