import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import javax.help.*;

/**
 * Die Klasse <CODE>PanelQuantisierung</CODE> reprsentiert die Seite
 * "Quantisierung" des Registerdialogs. Auf dieser Seite werden das Y-Teilbild, die
 * aktuelle Quantisierungsmatrix sowie der 8x8-Block jeweils vor und nach der
 * Quantisierung angezeigt. Beide 8x8-Blcke werden als 3D-Balkendiagramm visualisiert.
 * Der Benutzer verfgt ber folgende Interaktionsmglichkeiten:
 * Er kann einen 8x8-Block aus dem Bild durch Mausklick auswhlen, er kann jedes Element
 * der Quantisierungsmatrix einzeln verndern, er kann eine der vorgegebenen
 * Quantisierungsmatrizen auswhlen und diese mit einem Faktor skalieren.
 * @author  Raimund Specht
 */
public class PanelQuantisierung extends JPanel implements TableModelListener {
    private JPanel yImage;
    private JPanel bar3DDCT;
    private JPanel bar3DQuant;
    private JPanel dynPanel;
    private JPanel compressPanel;
    private JTable tableDCT;
    private JTable tableQMatrix;
    private JTable tableQuantBlock;
    private JComboBox matrixCombo;
    private JComboBox faktorCombo;
    private JProgressBar progressBar;
    private int lastMousePosX, lastMousePosY;

    /**
     * Der Konstruktor richtet das Panel ein. Er setzt als erste Quantisierungsmatrix die
     * im JPEG-Standard vorgegebene Matrix, reprsentiert durch
     * <CODE>JPEGParameter.STANDARD_QMATRIX</CODE>. Danach werden die statischen und die
     * dynamisch vernderlichen visuellen Komponenten eingebaut.
     * @author  Raimund Specht
     * @version 1-02.12.1999
     */
    public PanelQuantisierung() {
        setLayout(new BorderLayout());
        addStaticComponents();
        addDynamicComponents();
        validate();
    } // PanelQuantisierung()

    /**
     * Durch Aufruf dieser Methode werden die "dynamischen" Komponenten
     * neu dargestellt z.B. nach einer Benutzerinteraktion.
     * Die Neudarstellung erfolgt durch Lschen der alten und anschlieendem
     * Hinzufgen neuer Komponenten in <CODE>addDynamicComponents()</CODE>.
     * @author  Raimund Specht
     * @version 1-02.12.1999
     */
    public void update() {
        dynPanel.removeAll();
        compressPanel.removeAll();
        addDynamicComponents();
        validate();
    } // update()

    /**
     * Diese Methode wird vom ActionListener aufgerufen, wenn der Benutzer in einer der
     * beiden Komboboxen eine Auswahl durchgefhrt hat. Die Methode sorgt fr die
     * Neudarstellung der Bildschirmanzeige.
     * @author  Raimund Specht, Hari
     * @version 3-7.2.2000
     */
    private void comboBoxAuswahl() {
        int faktorAuswahl = 0;  // dummy init
        String faktorAuswahlStr = (String)faktorCombo.getSelectedItem();
        char chars[] = faktorAuswahlStr.toCharArray();
        boolean nurZiffern = true;
        
        for (int i=0; i<chars.length; i++)
            if (chars[i]<'0' || chars[i]>'9')
                nurZiffern = false;
        try {
            faktorAuswahl = (new Integer(faktorAuswahlStr)).intValue();
            if (faktorAuswahl < 1) {
                faktorAuswahl = 1;
                JOptionPane.showMessageDialog(null, "Der Wert muss zwischen 1 und 100 liegen!", "Fehlaaa!!!", JOptionPane.ERROR_MESSAGE);
                faktorCombo.setSelectedItem(Integer.toString(faktorAuswahl));
            }
            if (faktorAuswahl > 100) {
                faktorAuswahl = 100;
                JOptionPane.showMessageDialog(null, "Der Wert muss zwischen 1 und 100 liegen!", "Fehlaaa!!!", JOptionPane.ERROR_MESSAGE);
                faktorCombo.setSelectedItem(Integer.toString(faktorAuswahl));
            }
        } catch (NumberFormatException e) {
            if (nurZiffern)
                JOptionPane.showMessageDialog(null, "Der Wert muss zwischen 1 und 100 liegen!", "Fehlaaa!!!", JOptionPane.ERROR_MESSAGE);
            else
                JOptionPane.showMessageDialog(null, "Zur Information: eine Zahl besteht aus Ziffern von 0-9", "Fehlaaa!!!", JOptionPane.ERROR_MESSAGE);
            faktorAuswahl = FrameVisJPEG.vis.parameter.qFaktor;
            faktorCombo.setSelectedItem(Integer.toString(faktorAuswahl));
        }

        String matrixAuswahl = (String)matrixCombo.getSelectedItem();
        if (matrixAuswahl.equals("Standard")) {
            FrameVisJPEG.vis.parameter.qMatrixLum = (Matrix)JPEGParameter.STANDARD_QMATRIX_LUM.clone();
            FrameVisJPEG.vis.parameter.qMatrixChrom = (Matrix)JPEGParameter.STANDARD_QMATRIX_CHROM.clone();
        }
        if (matrixAuswahl.equals("Uniform")) {
            FrameVisJPEG.vis.parameter.qMatrixLum = (Matrix)JPEGParameter.UNIFORM_QMATRIX.clone();
            FrameVisJPEG.vis.parameter.qMatrixChrom = (Matrix)JPEGParameter.UNIFORM_QMATRIX.clone();
    	}
        if (matrixAuswahl.equals("Non-Uniform")) {
            FrameVisJPEG.vis.parameter.qMatrixLum = (Matrix)JPEGParameter.NONUNIFORM_QMATRIX.clone();
            FrameVisJPEG.vis.parameter.qMatrixChrom = (Matrix)JPEGParameter.NONUNIFORM_QMATRIX.clone();
        }

        FrameVisJPEG.vis.parameter.qMatrixLum.scale(faktorAuswahl);
        FrameVisJPEG.vis.parameter.qMatrixChrom.scale(faktorAuswahl);
        FrameVisJPEG.vis.parameter.qFaktor = faktorAuswahl;

        FrameVisJPEG.vis.update();
    } // comboBoxAuswahl()

    /**
     * In dieser Methoden werden smtliche "statischen" Komponenten
     * hinzugefgt, d.h. die Komponenten, die bei einem <CODE>update()</CODE>
     * nicht verndert werden mssen (Labels, Komboboxen usw.).
     * @author  Raimund Specht, Hari
     * @version 1-02.12.1999
     */
    private void addStaticComponents() {
        String[] matrixComboStrings = {"Standard", "Uniform", "Non-Uniform"};
        String[] faktorComboStrings = {"1", "2", "3", "5", 
        	                           "8", "10", "15", "20", 
        	                           "25", "30", "40", "50", 
        	                           "60", "70", "75", "80", 
        	                           "85", "90", "95", "100"};
        JLabel label;
        JButton vorButton;
        JButton infoButton;
        JButton backButton;    
        JPanel p;
        JPanel p2;

        p = new JPanel();
        p.setLayout(new BorderLayout());
        label = new JLabel(" Schritt 3: Quantisierung");
        p.add(label, BorderLayout.WEST);
        p2 = new JPanel();

        backButton = new JButton("Zurck");
        backButton.setMnemonic(KeyEvent.VK_Z);
        backButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                FrameVisJPEG.vis.setActivePanel(-1);
            }
        });
        p2.add(backButton);

        infoButton = new JButton("Info");
        infoButton.setMnemonic(KeyEvent.VK_I);
        if (JavaHelpSystem.helpSystemAvailable) {
            CSH.setHelpIDString(infoButton, "quantisierung");
            infoButton.addActionListener(new CSH.DisplayHelpFromSource(JavaHelpSystem.hb));
        } else {
            infoButton.setEnabled(false);
            infoButton.setToolTipText("HelpSet nicht gefunden.");
        }
        p2.add(infoButton);

        vorButton = new JButton("Weiter");
        vorButton.setMnemonic(KeyEvent.VK_W);
        vorButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                FrameVisJPEG.vis.setActivePanel(1);
            }
        });
        p2.add(vorButton);
        p.add(p2, BorderLayout.EAST);
        add(p, BorderLayout.NORTH);

        p = new JPanel();
        p2 = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
        p2.setLayout(new BorderLayout());
        label = new JLabel("Quantisiermatrix:");
        p.add(label);
        matrixCombo = new JComboBox(matrixComboStrings);
        matrixCombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                comboBoxAuswahl();
            }
        });
        p.add(matrixCombo);

        label = new JLabel("Qualittsfaktor:");
        p.add(label);
        faktorCombo = new JComboBox(faktorComboStrings);
        faktorCombo.setSelectedItem(Integer.toString(FrameVisJPEG.vis.parameter.qFaktor));
        faktorCombo.setEditable(true);
        faktorCombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                comboBoxAuswahl();
            }
        });
        p.add(faktorCombo);
        p2.add(p, BorderLayout.NORTH);
        
        compressPanel = new JPanel();
        compressPanel.setLayout(new BorderLayout());
        // Komponenten werden in addDynamicComponents() hinzugefgt
        p2.add(compressPanel, BorderLayout.SOUTH);
        add(p2, BorderLayout.EAST);

        dynPanel = new JPanel();
        dynPanel.setLayout(new GridLayout(3, 2));
        // Komponenten werden in addDynamicComponents() hinzugefgt
        add(dynPanel, BorderLayout.CENTER);
      } // addStaticComponents()

    /**
     * Diese Methode fgt alle "dynamischen" Komponenten zum Panel hinzu.
     * Bei einem <CODE>update()</CODE> mssen diese Komponenten zuerst
     * gelscht und dann neu eingefgt werden.
     * @author  Raimund Specht
     * @version 1-02.12.1999
     */
    private void addDynamicComponents() {
        JPanel p;
        JPanel p2;
        JLabel label;
        FloatBlock dctBlock;
        Block quantBlock;
        Matrix quantMatrix;

        quantMatrix = FrameVisJPEG.vis.parameter.qMatrixLum;
        dctBlock = FrameVisJPEG.vis.ourImage.getMarkedBlock().getDCT();
        quantBlock = dctBlock.getQuantisiert(quantMatrix);

        p = new JPanel();
        p2 = new JPanel();
        p.setLayout(new BorderLayout());
        label = new JLabel(" Y - Komponente:");
        p.add(label, BorderLayout.NORTH);

        yImage = new OurImage(FrameVisJPEG.vis.ourImage.getMarkedYImage());
        yImage.setCursor(new Cursor(Cursor.HAND_CURSOR));
        yImage.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                FrameVisJPEG.vis.ourImage.setMarker(e.getX(), e.getY());
                FrameVisJPEG.vis.update();
            }
            public void mouseEntered(MouseEvent e) {
                lastMousePosX = -10;  // unsichtbar
                lastMousePosY = -10;  // unsichtbar
            }
            public void mouseExited(MouseEvent e) {
                Graphics g = yImage.getGraphics();
                g.setXORMode(Color.black);
                g.setColor(Color.pink);
                g.drawRect(lastMousePosX, lastMousePosY, 7, 7);
            }
        });
        yImage.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseMoved(MouseEvent e) {
                Graphics g = yImage.getGraphics();
                g.setXORMode(Color.black);
                g.setColor(Color.pink);
                g.drawRect(lastMousePosX, lastMousePosY, 7, 7);
                g.drawRect(lastMousePosX=(e.getX()/8)*8, lastMousePosY=(e.getY()/8)*8, 7, 7);
            }
        });
        lastMousePosX = -10;  // Maus-Positions-Markierungs-Block unsichtbar machen
        lastMousePosY = -10;  // Maus-Positions-Markierungs-Block unsichtbar machen
        yImage.setToolTipText("Klicken, um 8x8-Block auszuwhlen");
        yImage.setPreferredSize(new Dimension(192, 144));
        p2.add(yImage);
        p.add(p2, BorderLayout.CENTER);
        dynPanel.add(p);

        p = new JPanel();
        p2 = new JPanel();
        p.setLayout(new BorderLayout());
        p.add(new JLabel(" Quantisierungsmatrix:"), BorderLayout.NORTH);
        tableQMatrix = quantMatrix.paintMatrix(GUIParameter.ACTIVE_BACKGROUND);
        tableQMatrix.setCursor(new Cursor(Cursor.TEXT_CURSOR));
        tableQMatrix.setRowSelectionAllowed(false);
        tableQMatrix.getModel().addTableModelListener(this);
        p2.add(tableQMatrix);
        p.add(p2, BorderLayout.CENTER);
        dynPanel.add(p);

        p = new JPanel();
        p2 = new JPanel();
        p.setLayout(new BorderLayout());
        p.add(new JLabel(" DCT - Matrix:"), BorderLayout.NORTH);
        tableDCT = dctBlock.paintMatrix(GUIParameter.INACTIVE_BACKGROUND);
        tableDCT.setRowSelectionAllowed(false);
        tableDCT.setEnabled(false);
        p2.add(tableDCT);
        p.add(p2, BorderLayout.CENTER);
        dynPanel.add(p);

        p = new JPanel();
        p2 = new JPanel();
        p.setLayout(new BorderLayout());
        p.add(new JLabel(" Quantisierte Matrix:"), BorderLayout.NORTH);
        tableQuantBlock = quantBlock.paintMatrix(GUIParameter.INACTIVE_BACKGROUND);
        tableQuantBlock.setRowSelectionAllowed(false);
        tableQuantBlock.setEnabled(false);
        p2.add(tableQuantBlock);
        p.add(p2, BorderLayout.CENTER);
        dynPanel.add(p);

        p = new JPanel();
        p2 = new JPanel();
        p.setLayout(new BorderLayout());
        p.add(new JLabel(" Visualisierung:"), BorderLayout.NORTH);
        bar3DDCT = new OurImage(dctBlock.paintBar3D(192, 144, Color.blue));
        bar3DDCT.setPreferredSize(new Dimension(192, 144));
        p2.add(bar3DDCT);
        p.add(p2, BorderLayout.CENTER);
        dynPanel.add(p);

        p = new JPanel();
        p2 = new JPanel();
        p.setLayout(new BorderLayout());
        p.add(new JLabel(" Visualisierung:"), BorderLayout.NORTH);
        bar3DQuant = new OurImage(quantBlock.paintBar3D(192, 144, Color.blue));
        bar3DQuant.setPreferredSize(new Dimension(192, 144));
        p2.add(bar3DQuant);
        p.add(p2, BorderLayout.CENTER);
        dynPanel.add(p);

/*        compressPanel.add(new JLabel("Datenreduktion:"), BorderLayout.NORTH);
        p = new JPanel();
        progressBar = new JProgressBar(JProgressBar.VERTICAL, 0, 100);
        progressBar.setValue(100);
        progressBar.setStringPainted(true);
        p.add(progressBar);
        compressPanel.add(p, BorderLayout.CENTER);*/
    } // addDynamicComponents()
    
    /**
     * Diese Methode wird aufgerufen, wenn der Benutzer einen Eintrag in der Quantisierungsmatrix
     * "von Hand" verndert hat. Die neue Matrix wird dann in die aktuellen <CODE>JPEGParameter</CODE>
     * geschrieben und es wird ein Neuzeichnen der Bildschirmanzeige veranlat.
     * @author  Raimund Specht
     * @version 1-03.12.1999
     * @param   e TableModelEvent, wird nicht beachtet
     */
    public void tableChanged(TableModelEvent e) {
        Matrix newMatrix;
        int i, j;
        
        newMatrix = new Matrix();
        for (i=0; i<8; i++)
            for (j=0; j<8; j++)
                newMatrix.matrix[i][j] = ((Integer)tableQMatrix.getValueAt(j, i)).intValue();
        FrameVisJPEG.vis.parameter.qMatrixLum = newMatrix;

        FrameVisJPEG.vis.update();
    } // tableChanged()
} // Klasse PanelQuantisierung