import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.help.*;
import java.net.*;


/**
 * Die Klasse <CODE>PanelZickZackScan</CODE> kapselt das Panel, das 
 * fr den Zick-Zack-Scan zustndig ist. Es zeigt die Y-Komponente des
 * Bildes, die Quantisierte Matrix des aktuellen 8x8 Blockes sowie 
 * zwei Statistiken (eine mit und eine ohne Anwendung des Zick-Zack-Scans).
 * @author Marco Gebbert
 * @version 1.02
 */

public class PanelZickZackScan extends JPanel {
	private JPanel yImage, dynpanel2,dynpanel;
	private JTable tableQuantBlock;
    private int lastMousePosX, lastMousePosY;
  

	/**
	 * Der Konstruktor fgt dem Panel die einzelnen Komponenten hinzu
	 * und instanziert es.
	 * @author Marco Gebbert
	 * @version 1.0
	 */
	public PanelZickZackScan() {
		setLayout(new BorderLayout());
		addKomponets(true);
		validate();
	}
  
  /**
   * Die Methode addKomponets fgt dem Panel alle ntigen Komponenten zu
   * 
   * @author Marco Gebbert
   * @version 1.0
   */

  private void addKomponets(boolean firststart) {
		JPanel jp, jp2,p,p2, bar2DZig, bar2DLin, zigImage;
		JLabel mylabel;
    JLabel label;
		JButton forwardButton, infoButton, backButton;
		Block quantBlock;
		FloatBlock dctBlock;
    Matrix quantMatrix;
    BlockVektor ZigVektor, LinVektor;
    Image rohImage = null;
    Image rohImage2 = null;
    ImageProducer ImProd;
    BufferedImage linIm,zigIm;
    
    // die beiden Bilder laden
    if (JPEG.applet == null) {
        rohImage = Toolkit.getDefaultToolkit().createImage("lin.gif");
        rohImage2 = Toolkit.getDefaultToolkit().createImage("zig.gif");
    } else {
        try {
            rohImage = JPEG.applet.getImage(new URL(JPEG.appletCodeBase, "lin.gif"));
            rohImage2 = JPEG.applet.getImage(new URL(JPEG.appletCodeBase, "zig.gif"));
        } catch(MalformedURLException ex) {}
    }
    linIm=loadImage(rohImage);
    zigIm=loadImage(rohImage2);
    
		if (firststart) {
			p = new JPanel();
			p2 = new JPanel();
			p.setLayout(new BorderLayout());
			mylabel = new JLabel(" Schritt 4: Zick-Zack-Scan");
			p.add(mylabel, BorderLayout.WEST);
			backButton = new JButton("Zurck");
              backButton.setMnemonic(KeyEvent.VK_Z);
			backButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					FrameVisJPEG.vis.setActivePanel(-1);
				}
			});
			p2.add(backButton);
			infoButton = new JButton("Info");
              infoButton.setMnemonic(KeyEvent.VK_I);
              if (JavaHelpSystem.helpSystemAvailable) {
                  CSH.setHelpIDString(infoButton, "zickzack");
                  infoButton.addActionListener(new CSH.DisplayHelpFromSource(JavaHelpSystem.hb));
              } else {
                  infoButton.setEnabled(false);
                  infoButton.setToolTipText("HelpSet nicht gefunden.");
              }
			p2.add(infoButton);
			forwardButton = new JButton("Weiter");
              forwardButton.setMnemonic(KeyEvent.VK_W);
			forwardButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					FrameVisJPEG.vis.setActivePanel(1);
				}
			});
			p2.add(forwardButton);
			p.add(p2, BorderLayout.EAST);
			add(p, BorderLayout.NORTH);
      dynpanel2 = new JPanel();
      dynpanel2.setLayout(new GridLayout(2,1));
      add(dynpanel2,BorderLayout.CENTER);
		}//if
    
    
  //folgende Komponenten mssen bei jedem Update neu gezeichnet werden
  dynpanel = new JPanel();
  
  dynpanel.setLayout(new GridLayout(1,2));
  
  
  
  quantMatrix = FrameVisJPEG.vis.parameter.qMatrixLum;
  dctBlock = FrameVisJPEG.vis.ourImage.getMarkedBlock().getDCT();
  quantBlock = dctBlock.getQuantisiert(quantMatrix);
  ZigVektor = quantBlock.getZigZagScan();
  LinVektor = quantBlock.getLinearScan();

  jp = new JPanel();
  jp2 = new JPanel();
  jp.setLayout(new BorderLayout());
  label = new JLabel(" Y - Komponente:");
  jp.add(label,BorderLayout.NORTH);
  yImage = new OurImage(FrameVisJPEG.vis.ourImage.getMarkedYImage());
  yImage.setCursor(new Cursor(Cursor.HAND_CURSOR));
  yImage.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
          FrameVisJPEG.vis.ourImage.setMarker(e.getX(), e.getY());
          FrameVisJPEG.vis.update();
      }
      public void mouseEntered(MouseEvent e) {
          lastMousePosX = -10;  // unsichtbar
          lastMousePosY = -10;  // unsichtbar
      }
      public void mouseExited(MouseEvent e) {
          Graphics g = yImage.getGraphics();
          g.setXORMode(Color.black);
          g.setColor(Color.pink);
          g.drawRect(lastMousePosX, lastMousePosY, 7, 7);
      }
  });
  yImage.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
          Graphics g = yImage.getGraphics();
          g.setXORMode(Color.black);
          g.setColor(Color.pink);
          g.drawRect(lastMousePosX, lastMousePosY, 7, 7);
          g.drawRect(lastMousePosX=(e.getX()/8)*8, lastMousePosY=(e.getY()/8)*8, 7, 7);
      }
  });
  lastMousePosX = -10;  // Maus-Positions-Markierungs-Block unsichtbar machen
  lastMousePosY = -10;  // Maus-Positions-Markierungs-Block unsichtbar machen
  yImage.setToolTipText("Klicken, um 8x8 Block auszuwhlen");
  yImage.setPreferredSize(new Dimension(192, 144));
  jp2.add(yImage,BorderLayout.CENTER);
  jp.add(jp2,BorderLayout.WEST);
  dynpanel.add(jp);
  
  jp = new JPanel();
  jp2 = new JPanel();
  jp.setLayout(new BorderLayout());
  jp.add(new JLabel(" Quantisierte Matrix:"), BorderLayout.NORTH);
  tableQuantBlock = quantBlock.paintMatrix(GUIParameter.INACTIVE_BACKGROUND);
  tableQuantBlock.setEnabled(false);
  jp2.add(tableQuantBlock);
  jp.add(jp2, BorderLayout.WEST);
  dynpanel.add(jp);
  dynpanel2.add(dynpanel);
  
  jp = new JPanel();
  jp2 = new JPanel();
  jp.setLayout(new BorderLayout(15,5));
  dynpanel = new JPanel();
  dynpanel.setLayout(new GridLayout(2,1));
  zigImage = new OurImage(linIm);
  zigImage.setPreferredSize(new Dimension(100,100));
  jp.add(new JLabel("Matrixwerte von links nach rechts und von oben nach unten:"),BorderLayout.NORTH);
  jp.add(zigImage,BorderLayout.WEST);
  bar2DLin=new OurImage(LinVektor.paintBar(400,100,Color.red));
  bar2DLin.setPreferredSize(new Dimension(400, 100));
  jp.add(bar2DLin,BorderLayout.CENTER);
  dynpanel.add(jp);
  
  jp = new JPanel();
  jp2 = new JPanel();
  jp.setLayout(new BorderLayout(15,5));
  zigImage = new OurImage(zigIm);
  zigImage.setPreferredSize(new Dimension(100,100));
  jp.add(new JLabel("Matrixwerte mit Zick-Zack-Scan Methode:"),BorderLayout.NORTH);
  jp.add(zigImage,BorderLayout.WEST);
  bar2DZig=new OurImage(ZigVektor.paintBar(400,100,Color.green));
  bar2DZig.setPreferredSize(new Dimension(400, 100));
  jp.add(bar2DZig,BorderLayout.CENTER);
  dynpanel.add(jp);
  dynpanel2.add(dynpanel);
  
  
  }//addKomponents

private BufferedImage loadImage(Image rohImage) {
        int width, height;
        Graphics2D g2D;
        MediaTracker mTracker;
        Canvas c = new Canvas();
        BufferedImage image;

        /* mit einem MediaTracker auf das Ende des Ladevorgangs warten */
        mTracker = new MediaTracker(c);
        mTracker.addImage(rohImage, 0);
        try {
            mTracker.waitForAll(); 
        } catch(InterruptedException e) {}
        
        if (mTracker.isErrorID(0)) {
            /* Fehler: "Fehlermeldungsbild" erzeugen */
            image = new BufferedImage(100, 100, BufferedImage.TYPE_INT_RGB);
        
            g2D = image.createGraphics();
            g2D.setColor(Color.white);
            g2D.drawString("Bild kann nicht geladen werden !", 5, 48);
            g2D.drawString("Image cannot be loaded !", 5, 96);
        } else { 
            /* alles ok: Image in ein BufferedImage konvertieren */
            width = rohImage.getWidth(this);
            height = rohImage.getHeight(this);
        
            image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
            g2D = image.createGraphics();
            g2D.drawImage(rohImage, 0, 0, this);
        }
        return image;
    }

	public void update() {
    dynpanel2.removeAll(); 
		addKomponets(false);  
    validate();
	} //update
} //PanelZickZackScan